﻿#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QMessageBox>
#include <QFileDialog>
#include <stdio.h>
#include <QTextCodec>

#include <selectbox.h>
#include <signalbox.h>
#include <aboutbox.h>
#include <setseqnum.h>
#include <bufferdlgbox.h>
#include <setlockbufferbox.h>
#include <setclientsizebox.h>
#pragma execution_character_set("utf-8")
#if defined(Q_OS_LINUX)
    int stricmp(const char * _Str1, const char * _Str2)
    {
        return strcasecmp(_Str1, _Str2);
    }

    int strnicmp(const char * _Str1, const char * _Str, size_t _MaxCount)
    {
        return strncasecmp(_Str1, _Str, _MaxCount);
    }
#endif


//--callback.c
BOOL CALLBACK exBeginCapture(HANDLE hBoard);
BOOL CALLBACK exBackDisplay(HANDLE hBoard, MLONG no);
BOOL CALLBACK exSetToScreen(HANDLE hBoard, long no);
BOOL CALLBACK exEndCapture(HANDLE hBoard);

BOOL CALLBACK exWriteAudioData(HANDLE hAudio, LPBYTE lpAudBuf, INT32 length);
BOOL CALLBACK exMsgCallback(HANDLE hStream, LPBYTE lpStreamBuf, long length);


long exGetTargetSize(HANDLE hBoard, TARGET tgt, short *width, short *heigth);
long exSetBitmapHeader(LPBITMAPINFOHEADER lpbi,short width,short height, short bits, short form);

extern BOOL				bDispToScreen;
extern LPBITMAPINFOHEADER	lpbi;
extern LPBYTE				lpdib;

static SelectBox *m_SelectDlg = nullptr;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    this->showMaximized();

    m_PaintView = new PaintView(this);
    this->setCentralWidget(m_PaintView);

    m_SelectDlg = new SelectBox;

    CreateMenu();

    bScrnCenter=exGetInitSet(bScrnCenter);

//#if defined(Q_OS_WIN32)
//    WritePrivateProfileString(L"Route", L"Created Win", L"", L"OkDemo.ini");
//#else
//    WritePrivateProfileString("Route","Created Win","","OkDemo.ini");
//#endif

#if defined(Q_OS_WIN32)
    WritePrivateProfileString(TEXT("Route"),TEXT("Created Win"),TEXT(""),TEXT("OkDemo.ini"));
#else
    WritePrivateProfileString("Route","Created Win","","OkDemo.ini");
#endif

    hBoard=exOpenCard();

//    {
//        HDC		hDC;

//        hDC=GetDC(GetDesktopWindow());
//        exSetPaletteToWnd(hBoard, hDC);
//        ReleaseDC(hWnd,hDC);
//    }
    //make mirror when convert
    okSetConvertParam(hBoard,CONVERT_MIRROR,bMakeMirror);
}

MainWindow::~MainWindow()
{
    delete m_SelectDlg;
    delete m_PaintView;

    delete ui;
}

bool MainWindow::eventFilter(QObject *obj, QEvent *event)
{
    if (obj == this && QEvent::Paint == event->type()) {
        update();
    }

    return QObject::eventFilter(obj,event);
}

void MainWindow::CreateMenu()
{
    //file

    //option

    //Freeze
    FREEZE = ui->menuBar->addAction(tr("&P停止"), this, SLOT(OnFreezeClicked()));
    FREEZE->setEnabled(false);

    //Active
    ACTIVE = ui->menuBar->addAction(tr("&A实时显"), this, SLOT(OnActiveClicked()));


    //Single
    ui->menuBar->addAction(tr("&S单帧采"), this, SLOT(OnSingleCapToClicked()));

    //help
    QAction *acTemp;
    QMenu *help = menuBar()->addMenu(tr("&H帮助"));
    acTemp = help->addAction(tr("&V -测试经缓存写显卡"));
    connect(acTemp, SIGNAL(triggered()), this, SLOT(OnTestVGACard()));
    acTemp = help->addAction(tr("&W -测试经缓存写窗口"));
    connect(acTemp, SIGNAL(triggered()), this, SLOT(OnTestDrawWnd()));
    acTemp = help->addAction(tr("&W -测试主控写窗口"));
    connect(acTemp, SIGNAL(triggered()), this, SLOT(OnTestMainWnd()));
    help->addSeparator();
    DISABLEACTIVE = help->addAction(tr("&D 禁直通实时显示"));
    DISABLEACTIVE->setCheckable(true);
    DISABLEACTIVE->setChecked(true);
    connect(DISABLEACTIVE, SIGNAL(triggered(bool)), this, SLOT(OnDisableActive(bool)));
    DISABLESYNCVERT = help->addAction(tr("&S 禁显示同步"));
    DISABLESYNCVERT->setCheckable(true);
    connect(DISABLESYNCVERT, SIGNAL(triggered()), this, SLOT(OnDisableSyncVert()));
    help->addSeparator();
    acTemp = help->addAction(tr("&A 分配缓存"));
    connect(acTemp, SIGNAL(triggered()), this, SLOT(OnSetAllocBuf()));
    acTemp = help->addAction(tr("&L 锁定缓存"));
    connect(acTemp, SIGNAL(triggered()), this, SLOT(OnLockBufSize()));
    acTemp = REGDEVDRIVER = help->addAction(tr("&R 安装设备驱动"));
    connect(acTemp, SIGNAL(triggered()), this, SLOT(OnRegDevDriver()));
    help->addSeparator();
    acTemp = help->addAction(tr("&S 信号源信息...\tF2"));
    connect(acTemp, SIGNAL(triggered()), this, SLOT(OnSignalClicked()));
    acTemp = help->addAction(tr("&T 测试外触发"));
    connect(acTemp, SIGNAL(triggered()), this, SLOT(OnExitTrigger()));
    acTemp = help->addAction(tr("&G 系统信息...\tF5"));  //, help, SLOT(OnAboutClicked));
    connect(acTemp, SIGNAL(triggered()), this, SLOT(OnAboutClicked()));
}


//file
void MainWindow::on_LOADFILE_triggered()
{
    exLoadImageFile(hBoard);

    sampwidth=(short)lpbi->biWidth;
    sampheight=(short)lpbi->biHeight;
    lScrnOffset=exSetToWndRectEx(hBoard, bScrnCenter);

    exAllocateDIBMemory(hBoard); //may be re-new size

    update();
    exDisplayDIBInfo();
}

void MainWindow::on_COPYTOCLIPB_triggered()
{
    exMoveToClipboard( lpbi, lpdib );
}

void MainWindow::on_SAVESCREEN_triggered()
{
    exSaveScreenFile(hBoard);
    update();
}

void MainWindow::on_SAVEBUFFER_triggered()
{
    exSaveBufferFile(hBoard);
    update();
}

void MainWindow::on_SETTOZERO_triggered()
{
    //clear screen dib and buffer 0
    LPBYTE		lpLinear;
    short		height;
    MLONG		stride;

    if(lpbi->biHeight)
        ZeroMemory(lpdib,lpbi->biSizeImage);
    update();

    lpLinear=(LPBYTE)okGetTargetInfo(hBoard,BUFFER,0, NULL,&height,&stride);
    /*
    {
        char tstr[80];
        sprintf(tstr, "lpLinear=%p, stride =%d height=%d", lpLinear, stride, height);
        MessageBox(NULL,tstr,NULL,MB_OK);
    }
    */
    if(lpLinear)
        ZeroMemory(lpLinear,stride*height);
}

void MainWindow::on_SETTOBAND_triggered()
{
    //set gray and color bands to dib and buffer 0
    LPBYTE	lpLinear;
    short	height,width;
    MLONG	stride;
    BLOCKINFO	blk;

    //set dib
    lpLinear=(LPBYTE)okGetTargetInfo(hBoard,BUFFER,0, &width,&height,&stride);
    if(lpLinear && (lpbi->biBitCount!=24) ) {
        exSetTestBands(lpLinear,(short)(stride/width*8),height, stride); //buffer
        exSetDataToDIB(BUFFER, 0, lpbi, lpdib);
    }
    else { //dib is 24 bits mode
        //set dib first
        exSetTestBands(lpdib, (short)lpbi->biBitCount, (short)lpbi->biHeight, lpbi->biSizeImage/lpbi->biHeight);
        exConvertBitmap(lpbi,(LPSTR)lpdib);
        ZeroMemory(&blk,sizeof(BLOCKINFO));
        //get image to app buffer from tgt
        blk.lpBits=lpdib;
        blk.iWidth=(short)lpbi->biWidth;
        blk.iHeight=-(short)lpbi->biHeight; //!!!to invert y
        blk.iBitCount=lpbi->biBitCount;
        blk.lBlockStep=(short)lpbi->biSizeImage; //must be set
        blk.iHiStep=HIWORD(lpbi->biSizeImage);
        if(lpbi->biCompression==BI_BITFIELDS ) {//
            DWORD	*lpMask;
            lpMask=(DWORD *)((LPBYTE)lpbi+lpbi->biSize);
            if(lpMask[1]==0x03e0) //555
                blk.iFormType=FORM_RGB555;
            else if(lpbi->biBitCount==16)
                blk.iFormType=FORM_RGB565;
            else if(lpbi->biBitCount==32)
                blk.iFormType=FORM_RGB8888;
        }
        okConvertRect(hBoard,BUFFER,0,(TARGET)&blk,0,1);
    }

    update();
}

void MainWindow::on_ZEROALLBUFFER_triggered()
{
    //clear screen dib and buffer 0
    LPBYTE		lpLinear;
    MWORD		dwSize;

    okGetBufferSize(hBoard,(void **)&lpLinear,&dwSize);
    //okGetAvailBuffer(&lpLinear,&dwSize);
    ZeroMemory(lpLinear,dwSize);

    if(lpbi->biHeight)
       ZeroMemory(lpdib,lpbi->biSizeImage);
    update();
}

void MainWindow::on_PRINTPIC_triggered()
{
    exPrintReport((HWND)winId(), lpbi, lpdib);
}

void MainWindow::on_LOADCONFIGFILE_triggered()
{
    exLoadCfgFile(hBoard);

    InitSetMenu(hBoard);

    exGetBitmapHeader(hBoard, BUFFER, lpbi); //set-lpbi
    //get current size
    sampwidth=(short)lpbi->biWidth;
    sampheight=(short)lpbi->biHeight;
    lScrnOffset=exSetToWndRectEx(hBoard, bScrnCenter);

    exAllocateDIBMemory(hBoard); //may be re-new size
    update();
}

void MainWindow::on_SAVECONFIGFILE_triggered()
{
    QString qsFileName;
    int     iFilterIndex = 0;
    QStringList szFilter;
    QFileDialog* fd = new QFileDialog(this);

    szFilter <<  "Config file with full params. (*.OKF)"
             <<  "Config file just for sample (*.OKS)"
             <<  "Config file just for output (*.OKO)"
             <<  "CamLink Mode Config File (*.CLF)"
             <<  "IO Config File (*.IOF)";
    fd->setAcceptMode(QFileDialog::AcceptSave);
    fd->setNameFilters(szFilter); //设置文件过滤器
    if ( fd->exec() == QDialog::Accepted )   //如果成功的执行
    {
        int i, total;

        total = szFilter.size();

        for (i=0; i<total; i++)
        {
           if (szFilter.at(i) == fd->selectedNameFilter())
           {
               iFilterIndex = i;
               break;
           }
        }

        switch (iFilterIndex)
        {
        case 0:
           fd->setDefaultSuffix("okf");
           break;
        case 1:
           fd->setDefaultSuffix("oks");
           break;
        case 2:
           fd->setDefaultSuffix("oko");
           break;
        case 3:
           fd->setDefaultSuffix("clf");
           break;
        case 4:
           fd->setDefaultSuffix("iof");
           break;
        default:
           fd->setDefaultSuffix("okf");
           break;
        }

        qsFileName = fd->selectedFiles()[0];            //取第一个文件名
        QTextCodec::setCodecForLocale(QTextCodec::codecForName("GBK"));
        QByteArray ba = qsFileName.toLocal8Bit();
        okSaveConfigFile(hBoard, ba.data());
    }
    else
    fd->close();

    delete fd;
}

void MainWindow::on_EXITOKDEMO_triggered()
{
    close();
}


//option
void MainWindow::on_SETUPVIDEO_triggered()
{
#if defined(Q_OS_WIN32)
    //restore interrupt set
    okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,intrMISCCONTROL|
            (okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,-1)&~0x3));

    //SetWindowLong(hWnd,GWL_USERDATA,1); //set dlg to modeless
    okOpenSetParamDlg(hBoard, (HWND)this->winId());
    //may be initialized
    intrMISCCONTROL=okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,-1)&0x3;

    //dest(buffer) rect may be changed
    exGetBitmapHeader(hBoard, BUFFER, lpbi); //set-lpbi as BUFFER
    //get current size
    sampwidth=(short)lpbi->biWidth;
    sampheight=(short)lpbi->biHeight;

    lScrnOffset=exSetToWndRectEx(hBoard,bScrnCenter);
    if( okGetBufferSize( hBoard, nullptr, nullptr) <2 ) {
        QMessageBox::warning(this, "     Some operation can not run correctly !\n Because BUFFER is not enough for current image size. "," Warning !");
    }

    exAllocateDIBMemory(hBoard); //may be re-new size
    //okSetTargetRect(hBoard, SCREEN, NULL); //validate rect to capture

    //get current mask mode
    bMaskMode=(short)okEnableMask(hBoard,-1);
    if(bMaskMode)
        ui->ENABLEMASK->setEnabled(true);
    else
        ui->ENABLEMASK->setEnabled(false);

    if( !okSetConvertParam(hBoard,CONVERT_UPRIGHT,-1)) { //rotate offed
        ui->UPTORIGHT->setChecked(false);
        ui->UPTOLEFT->setChecked(false);
    }
    update();
#endif
}

void MainWindow::on_CAPSEQUENCE_triggered()
{
#if defined(Q_OS_WIN32)
    okOpenSeqCaptureDlg(hBoard, (HWND)this->winId());
    //dest(buffer) rect may be changed
    exGetBitmapHeader(hBoard, BUFFER, lpbi); //set-lpbi
    //get current size
    sampwidth=(short)lpbi->biWidth;
    sampheight=(short)lpbi->biHeight;
    lScrnOffset=exSetToWndRectEx(hBoard, bScrnCenter);
#endif
}

void MainWindow::on_FILLINGLUT_triggered()
{
#if defined(Q_OS_WIN32)
    okOpenSetLUTDlg(hBoard, (HWND)winId(), NULL);
#endif
}

void MainWindow::on_CENTERSCREEN_triggered()
{
    if(ui->CENTERSCREEN->isChecked()) {
        ui->CENTERSCREEN->setChecked(false);
        bScrnCenter=0;
    }
    else {
        ui->CENTERSCREEN->setChecked(true);
        bScrnCenter=1;
        ui->MATCHCLIENT->setChecked(false);
        bMatchClient=0;
    }
    lScrnOffset=exSetToWndRectEx(hBoard, bScrnCenter);
    update();
}

void MainWindow::on_ENABLEMASK_triggered()
{
    if(ui->ENABLEMASK->isChecked()) {
        ui->ENABLEMASK->setChecked(false);
        bMaskMode=0; //disable mask
    }
    else {
        ui->ENABLEMASK->setChecked(true);
        bMaskMode=1; //positive mask
    }
    okEnableMask(hBoard,bMaskMode); //set mask
}

void MainWindow::on_MAKEXMIRROR_triggered()
{
    if(ui->MAKEXMIRROR->isChecked()) {
        ui->MAKEXMIRROR->setChecked(false);
        bMakeMirror &=2;
    }
    else {
        ui->MAKEXMIRROR->setChecked(true);
        bMakeMirror |=1;
    }
    okSetConvertParam(hBoard,CONVERT_MIRROR,bMakeMirror);
}

void MainWindow::on_MAKEYMIRROR_triggered()
{
    if(ui->MAKEYMIRROR->isChecked()) {
        ui->MAKEYMIRROR->setChecked(false);
        bMakeMirror &=1;
    }
    else {
        ui->MAKEYMIRROR->setChecked(true);
        bMakeMirror |=2;
    }
    okSetConvertParam(hBoard,CONVERT_MIRROR,bMakeMirror);
}

void MainWindow::on_UPTORIGHT_triggered()
{
    short	bChange=1;
    short	bCheckRight=ui->UPTORIGHT->isChecked();
    short	bCheckLeft=ui->UPTOLEFT->isChecked();

    if(bCheckLeft) {
        ui->UPTOLEFT->setChecked(false);
        ui->UPTORIGHT->setChecked(true);
        okSetConvertParam(hBoard,CONVERT_UPRIGHT,1); //to right
        bChange=0;
    }
    else if(bCheckRight) {
        ui->UPTORIGHT->setChecked(false);
        okSetConvertParam(hBoard,CONVERT_UPRIGHT,0); //un to
    }
    else { //new check
        ui->UPTORIGHT->setChecked(true);
        okSetConvertParam(hBoard,CONVERT_UPRIGHT,1); //up to right
    }

    if( bChange)  //exchange w and h
    { //treat rotate
        long	temp=lpbi->biWidth;

        lpbi->biWidth=lpbi->biHeight;
        lpbi->biHeight=temp;

        sampwidth=(WORD)lpbi->biWidth;
        sampheight=(WORD)lpbi->biHeight;
        exSetBitmapHeader(lpbi, (short)lpbi->biWidth, (short)lpbi->biHeight, (short)lpbi->biBitCount, 0 );
    }
    update();
}

void MainWindow::on_UPTOLEFT_triggered()
{
    short	bChange=1;
    short	bCheckRight=ui->UPTORIGHT->isChecked();
    short	bCheckLeft=ui->UPTOLEFT->isChecked();

    if (bCheckRight) {
        ui->UPTORIGHT->setChecked(false);
        ui->UPTOLEFT->setChecked(true);
        okSetConvertParam(hBoard,CONVERT_UPRIGHT,2); //to left
        bChange=0;
    }
    else if(bCheckLeft) {
        ui->UPTOLEFT->setChecked(false);
        okSetConvertParam(hBoard,CONVERT_UPRIGHT,0); //un to
    }
    else { //new check
        ui->UPTOLEFT->setChecked(true);
        okSetConvertParam(hBoard,CONVERT_UPRIGHT,2); //up to left
    }

    if( bChange)  //exchange w and h
    { //treat rotate
        long	temp=lpbi->biWidth;

        lpbi->biWidth=lpbi->biHeight;
        lpbi->biHeight=temp;

        sampwidth=(WORD)lpbi->biWidth;
        sampheight=(WORD)lpbi->biHeight;
        exSetBitmapHeader(lpbi, (short)lpbi->biWidth, (short)lpbi->biHeight, (short)lpbi->biBitCount, 0 );
    }
    update();
}

void MainWindow::on_NORMAINLUT_triggered()
{
    exFillInputLUT(hBoard, 0);
}

void MainWindow::on_INVERSEINLUT_triggered()
{
    exFillInputLUT(hBoard, 1);
}

void MainWindow::on_MATCHCLIENT_triggered()
{
    if(ui->MATCHCLIENT->isChecked()) {
        ui->MATCHCLIENT->setChecked(false);
        bMatchClient=0;
    }
    else {
        //RECT	rect;

        ui->MATCHCLIENT->setChecked(true);
        bMatchClient=1;
        ui->CENTERSCREEN->setChecked(false);
        bScrnCenter=0;

        //SetRect(&rect,0,0,lpbi->biWidth,lpbi->biHeight);
        //okSetTargetRect(hBoard, BUFFER, &rect);
        //okSetTargetRect(hBoard, VIDEO, &rect);
    }
    lScrnOffset=exSetToWndRectEx(hBoard, bScrnCenter);
    update();
}

void MainWindow::on_SETMATCHSIZE_triggered()
{
    SetClientSizeBox m_dlg;

    m_dlg.exec();

    update();
}

void MainWindow::on_REPLAYBUFFER_triggered()
{
#if defined(Q_OS_WIN32)
    total=(short)okGetBufferSize(hBoard, nullptr, nullptr);
    hwndPlay=okOpenReplayDlgEx(hBoard, (HWND)winId(), BUFFER,total, lpbi, lpdib);
#endif
}

void MainWindow::on_REPLAYMEMORY_triggered()
{
#if defined(Q_OS_WIN32)
    int	num;
    num=dwMaxMemSize/exGetFrameSize(hBoard);
    if(lpbi)
        hwndPlay=okOpenReplayDlgEx(hBoard, (HWND)winId(), (TARGET)lpMemory,num, lpbi, lpdib);
#endif
}

void MainWindow::on_REPLAYFILE_triggered()
{
#if defined(Q_OS_WIN32)
    int num;
    char	szFileName[128]="\0";
    if( !exGetLoadName(szFileName) )
        return;

    if( !stricmp(strrchr(szFileName,'.'),".seq") ||
            !stricmp(strrchr(szFileName,'.'),".m2v") ||
            !stricmp(strrchr(szFileName,'.'),".mpg") ||
            !stricmp(strrchr(szFileName,'.'),".avi") )
            {
        //just get num
        //num=okLoadImageFile(hBoard, szFileName, 0, SCREEN, 0, 1);
        num=0; //unkonw
    } else {
        SetSeqNum   m_dlg;
        m_dlg.exec();
        num = m_dlg.GetNum();
    }


    //lpDibPlay=okOpenReplayDlg(hBoard, hWnd, (TARGET)szFileName, lNumImage);

    okLoadImageFile(hBoard,szFileName,0,(TARGET)BUFFER,0,0);//no=0
    exAllocateDIBMemory(hBoard); //may be re-new size

    hwndPlay=okOpenReplayDlgEx(hBoard, (HWND)winId(), (TARGET)szFileName, num, lpbi, lpdib);
#endif
}

void MainWindow::on_BUFFERTOSCREEN_triggered()
{
    exSetDataToDIB(BUFFER, 0, lpbi, lpdib);
    update();
}

void MainWindow::on_SCREENTOBUFFER_triggered()
{
    {	BLOCKINFO	blkinfo;

        ZeroMemory(&blkinfo,sizeof(BLOCKINFO)); //do zero

        blkinfo.lpBits=lpdib;
        blkinfo.iBitCount=lpbi->biBitCount;
        blkinfo.iWidth=(short)lpbi->biWidth;
        blkinfo.iHeight=-(short)lpbi->biHeight;

        okConvertRect(hBoard,BUFFER,0,(TARGET)(&blkinfo),0,1);
    }
}

void MainWindow::on_BUFFER0TOBUFFER1_triggered()
{
    okTransferRect(hBoard, BUFFER, 1, BUFFER, 0, 1);
}

void MainWindow::on_BUFFER1TOBUFFER0_triggered()
{
    okConvertRect(hBoard,BUFFER,0,BUFFER,1,1);
}

void MainWindow::on_BUFFERTOFRAME_triggered()
{
    FREEZE->setEnabled(true);
    //if( okGetTypeCode(hBoard,NULL)==OK_M60 )
    //{ //special for M60 with model frame
        //can only by TransferRect for M60
    if(okTransferRect(hBoard,FRAME,0,BUFFER,0,1)>0) //fast or hard
    {
        Sleep(80);
        //okTransferRect(hBoard,FRAME,0,BUFFER,0,1);

    }
    else
        okConvertRect(hBoard,FRAME,0,BUFFER,0,1);
}

void MainWindow::on_FRAMETOBUFFER_triggered()
{
    //special for RGB30 with frame in board
    if( okGetTypeCode(hBoard, nullptr)==OK_RGB30 )
        okTransferRect(hBoard,BUFFER,0,FRAME,0,1);
    else //just for C70 with frame in board
        okConvertRect(hBoard,BUFFER,0,FRAME,0,1);
}

void MainWindow::on_FRAMETOSCREEN_triggered()
{
    //special for RGB30 with frame in board
    if( okGetTypeCode(hBoard, nullptr)==OK_RGB30 )
        okTransferRect(hBoard,SCREEN,0,FRAME,0,1);
    else if(okConvertRect(hBoard,SCREEN,0,FRAME,0,1)>0) {
        //take buffer size
        exGetBitmapHeader(hBoard, BUFFER, lpbi);
        exSetDataToDIB(SCREEN, 0, lpbi, lpdib);
    }
}

void MainWindow::on_SELECTCARD_triggered()
{
    m_SelectDlg->SetBoard(hBoard);
    m_SelectDlg->SetCurrentNo(iCurrUsedNo);

    if (m_SelectDlg->exec() == QDialog::Accepted)
    {
        if (iCurrUsedNo != m_SelectDlg->GetIndex())
        {
            exCloseThisBoard(hBoard);

            iCurrUsedNo = m_SelectDlg->GetIndex();
            hBoard=exOpenCard();
            bActive=0;
        }
        lScrnOffset=exSetToWndRectEx(hBoard, bScrnCenter);
    }
}

//Capture
void MainWindow::on_CAPTOBUFFER_triggered()
{
    int num;

    OnFreezeClicked();
    FREEZE->setEnabled(true);

    num=okGetBufferSize(hBoard, nullptr, nullptr);//

    SetSeqNum m_SSNDlg;

    m_SSNDlg.SetNum(num);
    if (m_SSNDlg.exec() == QDialog::Accepted)
    {
        num = m_SSNDlg.GetNum();
    }
    else
        return;

    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);

    okCaptureTo(hBoard,BUFFER,0,num);//
}

void MainWindow::on_LOOPTOBUFFER_triggered()
{
    OnFreezeClicked();
    FREEZE->setEnabled(true);

    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);

    okCaptureTo(hBoard,BUFFER,0,-1);//-1: loop infiniately
}

void MainWindow::on_SEQCAPTOBUF_triggered()
{
    int num;

    OnFreezeClicked();
    FREEZE->setEnabled(true);

    num=okCaptureSequence(hBoard,0,0); //just get num for sequence, another way to get num

    SetSeqNum m_SSNDlg;

    m_SSNDlg.SetNum(num);
    if (m_SSNDlg.exec() == QDialog::Accepted)
    {
        num = m_SSNDlg.GetNum();
    }
    else
        return;

    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);

    okCaptureSequence(hBoard,0,num);// num
}

void MainWindow::on_LOOPSEQCAPTOBUF_triggered()
{
    OnFreezeClicked();
    FREEZE->setEnabled(true);

    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);

    okCaptureSequence(hBoard,0,-1);//
}

void MainWindow::on_CAPTOMEMORY_triggered()
{
    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    //
    //okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,1);
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);

    if(lpbi) {
        int			num;
        BLOCKINFO	blk;

        ZeroMemory(&blk,sizeof(BLOCKINFO));
        blk.iType=BLKHEADER;
        blk.lpBits=lpMemory;

        //blk.iFormType=FORM_YUV422;
        //blk.iBitCount==16;

        //blk.iBitCount=lpbi->biBitCount;
        //blk.iWidth=(short)lpbi->biWidth;
        //blk.iHeight=(short)lpbi->biHeight;

        // just change buffer to supported, if it is now not supported
        okCaptureTo(hBoard,BUFFER,-1,0); // not really to capture
        num=dwMaxMemSize/exGetFrameSize(hBoard);
        lpMemory[0]=0;
        //okCaptureByBuffer(hBoard,(TARGET)lpMemory,0,num);
        if(num>0)
            okCaptureByBuffer(hBoard,(TARGET)&blk,0,num);
    }
}

void MainWindow::on_CAPTOSEQFILE_triggered()
{
    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    //
    //okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,1);
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);

    {
        char	szFileName[128]="\0";
        if( !exGetSaveName(szFileName) )
            return;

        SetSeqNum m_SSNDlg;

        if (m_SSNDlg.exec() == QDialog::Accepted)
        {
            long	quality=0;

            if( !stricmp(strrchr(szFileName,'.'),".jpg") ) //jpg
                quality=50;
            okCaptureByBufferEx(hBoard,quality,(TARGET)szFileName,0,m_SSNDlg.GetNum());
        }
        else {
            FREEZE->setEnabled(false);
        }
    }
}

void MainWindow::on_CAPTOINDIRECT_triggered()
{
    ui->CAPTOINDIRECT->setChecked(true);
    ui->INTERTOINDIRECT->setChecked(false);

    OnFreezeClicked();
    FREEZE->setEnabled(true);
    //restore to normal (system palette)
    okSetConvertParam(hBoard, CONVERT_PALETTE, (LPARAM)NULL);

    MLONG lret=okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,-1)&~0x3;
    okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,lret); //normal

    //okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,3); //normal

    //way 2 can support menu overlay, but too slow for NT
    bDispToScreen=2;
    okSetSeqCallback(hBoard, exBeginCapture, exBackDisplay, exEndCapture);
        //just capture to buffer

    if( okCaptureByBuffer(hBoard,BUFFER,0,0)==(HANDLE)1) //0: cont.
        //no enough buffer.
        QMessageBox::warning(this, "\n This need at least two frame size! \n\t\t Try to increase bigger size of BUFFER .",
            " No enough BUFFER can be used ");
}

void MainWindow::on_INTERTOINDIRECT_triggered()
{
    ui->CAPTOINDIRECT->setChecked(false);
    ui->INTERTOINDIRECT->setChecked(true);

    OnFreezeClicked();
    FREEZE->setEnabled(true);
    //restore to normal (system palette)
    okSetConvertParam(hBoard, CONVERT_PALETTE, (LPARAM)NULL);
    {
        MLONG lret=okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,-1)&~0x3;
        okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,lret|1); //forced interrupt controlled
    }

    //okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,3); //normal

    //way 2 can support menu overlay, but too slow for NT
    bDispToScreen=2;;
    okSetSeqCallback(hBoard, exBeginCapture, exBackDisplay, exEndCapture);
        //just capture to buffer

    if( okCaptureByBuffer(hBoard,BUFFER,0,0)==(HANDLE)1) //0: cont.
        //no enough buffer.
        QMessageBox::information(this,""
            "\n This need at least two frame size! \n\t\t Try to increase bigger size of BUFFER .",
            " No enough BUFFER can be used ");

}

void MainWindow::on_CAPTOPSEUDO_triggered()
{
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    bDispToScreen=3; //pseudo back display
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);
    //just capture to buffer
    okCaptureByBuffer(hBoard,BUFFER,0,0);//0: cont.
}

void MainWindow::on_CAPTODIRECT_triggered()
{
    okStopCapture(hBoard);
    //must not to upright
    ui->UPTORIGHT->setChecked(false);
    ui->UPTOLEFT->setChecked(false);
    okSetConvertParam(hBoard,CONVERT_UPRIGHT,0); //un to upright

    lScrnOffset=exSetToWndRectEx(hBoard,bScrnCenter);
    if( okCaptureToScreen(hBoard)<=0 ) {
        long	errcode=okGetLastError();

        if(errcode==ERR_FORMNOTSUPPORT) {
            QMessageBox::information(this,""
                "\tLook up which format this card support ? \n\t\t Try 256 color or 24 bits color mode.",
                "This card does not support current display mode !",MB_OK);
        }
        else {//
            char	str[100];
            sprintf(str,"Some thing wrong ! Error Code = %d",errcode);
            QMessageBox::information(this, str, "Error",MB_OK);
        }

    }
    else {
#if (0)
        MSG		msg;

        //exEnableItem(hWnd, IDM_ACTIVE, FALSE);
        FREEZE->setEnabled(true);
        bActive=1;

        SetCapture(hWnd);
        while(1) {

            //PeekMessage(&msg, NULL, 0, 0, PM_REMOVE);
            GetMessage(&msg, hWnd, 0, 0);
            if( msg.message==WM_KEYDOWN ||
                msg.message==WM_LBUTTONDOWN ||
                msg.message==WM_NCLBUTTONDOWN ) {
                    break;
            }
        }
        ReleaseCapture();
//						UnlockCapScreen(hBoard);

        OnFreezeClicked();
        update();
        //UpdateWindow(NULL);
#endif
    }
}

void MainWindow::on_CAPTOFOREVER_triggered()
{
    //must not to match client rect
    ui->MATCHCLIENT->setChecked(false);
    bMatchClient=0;

    //must not to upright
    ui->UPTORIGHT->setChecked(false);
    ui->UPTOLEFT->setChecked(false);
    okSetConvertParam(hBoard,CONVERT_UPRIGHT,0); //un to upright


    lScrnOffset=exSetToWndRectEx(hBoard,bScrnCenter);
    //UpdateWindow(hWnd);
    okStopCapture(hBoard); //confirm


    //specail capture to frame
    if( okGetTypeCode(hBoard, nullptr)==OK_C70 ) {
        okCaptureTo(hBoard,FRAME,0,0);
        //exEnableItem(hWnd, IDM_ACTIVE, FALSE);
        FREEZE->setEnabled(true);
    } //normal to screen
    else if( okCaptureToScreen(hBoard)<=0 ) { //not support
        long	errcode=okGetLastError();

        if(errcode==ERR_FORMNOTSUPPORT) {

            char	str[120],str1[120];
            long	bits;

            bits=okSetCaptureParam(hBoard,CAPTURE_SCRRGBFORMAT,-1);
            sprintf(str," Direct Capture To Screen");
            okGetTypeCode(hBoard,str1);
            sprintf(str1+strlen(str1)," does not support current %d bits mode !\n\n"
                "Please try other display mode.",HIWORD(bits));
            QMessageBox::information(this, str1,str,MB_OK);

            //MessageBox(hwnd,""
            //	"\tLook up which format this card support ? \n\t\t Try 256 color or 24 bits color mode.",
            //	"This card does not support current display mode !",MB_OK);
        }
        else {//
            char	str[100];
            sprintf(str,"Some thing wrong ! Error Code = %d",errcode);
            QMessageBox::information(this, str, "Error",MB_OK);
        }
    }
    else {  //normal capture alive to screen
        //exEnableItem(hWnd, IDM_ACTIVE, FALSE);
        FREEZE->setEnabled(true);
        bActive=2;
    }
}

void MainWindow::on_CONTTOBUFFER0_triggered()
{
    FREEZE->setEnabled(true);
    okCaptureTo(hBoard,BUFFER,0,0);
    if(ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
}

void MainWindow::on_CONTTOBUFFER1_triggered()
{
    FREEZE->setEnabled(true);
    okCaptureTo(hBoard,BUFFER,1,0);
    if(ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
//    exCreateWatching(hBoard, hWnd, 2);
}

void MainWindow::on_CAPTOFRAME_triggered()
{
    FREEZE->setEnabled(true);
    okCaptureTo(hBoard,FRAME,0,0); //cont.
}

void MainWindow::on_MULCHANCAP_triggered()
{

}

void MainWindow::on_MULCHANCAPSUB_triggered()
{

}

void MainWindow::on_ASYNCMULCAP_triggered()
{

}

void MainWindow::on_SYNCMULCAP_triggered()
{

}

void MainWindow::on_CAPTUREAUDIO_triggered()
{
    exCaptureAudioFile(hBoard, (HWND)winId());
}

void MainWindow::on_CAPTUREMPEG2_triggered()
{
    static  int flag=0;


    if(flag==0) {
        ui->CAPTUREMPEG2->setText(tr("&G 停止采集"));
        flag=1;
        if(!exCaptureStreamFile(hBoard, (HWND)winId()))
            flag=0;
    }
    else {
        exEndCaptureStream(hStream);
        flag=0;
        ui->CAPTUREMPEG2->setText(tr("&G 采集压缩图象"));
    }
}

//Display
void MainWindow::on_DISPFROMBUFFER_triggered()
{
    long	num;

    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    //bDispToScreen=0;
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);
    num=okGetBufferSize(hBoard, nullptr, nullptr);//


    okPlaybackFrom(hBoard,BUFFER,0,num);//
}

void MainWindow::on_LOOPFROMBUFFER_triggered()
{
    long	num;

    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    //bDispToScreen=0;
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);
    num=okGetBufferSize(hBoard, nullptr, nullptr);//


    okPlaybackFrom(hBoard,BUFFER,0,-num); //<0: loop infinitly
}

void MainWindow::on_INTERFROMBUFFER_triggered()
{
    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    //bDispToScreen=0;
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);


    okPlaybackSequence(hBoard,0,-1);//
}

void MainWindow::on_DISPFROMMEMORY_triggered()
{
    long	num;

    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    //bDispToScreen=0;
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);
    if(lpbi) {

        num=dwMaxMemSize/exGetFrameSize(hBoard);
        if(num>0)
            okPlaybackByBuffer(hBoard,(TARGET)lpMemory,0,-num);
    }
}

void MainWindow::on_DISPFROMFILE_triggered()
{
    long	num;

    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;
    //bDispToScreen=0;
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);

    {
        char	szFileName[128]="\0";
        if( !exGetLoadName(szFileName) )
            return;

        if( !stricmp(strrchr(szFileName,'.'),".seq") ||
                !stricmp(strrchr(szFileName,'.'),".m2v") ||
                !stricmp(strrchr(szFileName,'.'),".mpg") ||
                    !stricmp(strrchr(szFileName,'.'),".avi") )
            //just get num
            num=okLoadImageFile(hBoard, szFileName, 0, SCREEN, 0, 0);
        else {
            SetSeqNum m_dlg;

            if (m_dlg.exec() == QDialog::Accepted)
            {
                num = m_dlg.GetNum();
            }
        }
        okPlaybackByBuffer(hBoard,(TARGET)szFileName,0,-num);
    }
}

void MainWindow::on_CAPTOMONITOR_triggered()
{
    FREEZE->setEnabled(true);
    okCaptureTo(hBoard,MONITOR,0,0);//0: cont.
}

void MainWindow::on_DISPFROMFRAME_triggered()
{
    FREEZE->setEnabled(true);
    okPlaybackFrom(hBoard,FRAME,0,0); //cont.
}

void MainWindow::on_NORMALLUT_triggered()
{
    exFillOutLUT(hBoard, 0);
}

void MainWindow::on_INVERSELUT_triggered()
{
    exFillOutLUT(hBoard, 1);
}

void MainWindow::on_ABSOLUTELUT_triggered()
{
    exFillOutLUT(hBoard, 2);
}

void MainWindow::on_ABSOLUTEDSA_triggered()
{
    exFillOutLUT(hBoard, 3);
}

//Freeze
void MainWindow::OnFreezeClicked()
{
    if(hAudio)  //audio capturing
    {
        exEndCaptureAudio(hAudio);
        hAudio = nullptr;
    }

    if(hLoopThread) {
        TerminateThread(hLoopThread,0);
        hLoopThread = nullptr;
    }
    exStopMulCapture(); //to stop in case mul-capturing

    okStopCapture(hBoard);

    update();
    if(bActive)
    {
        //take buffer size
        exGetBitmapHeader(hBoard, BUFFER, lpbi);
        exSetDataToDIB(SCREEN, 0, lpbi, lpdib);
        //exGetBitmapData(hBoard, SCREEN, 0, lpbi, lpdib);
        exDisplayDIBInfo();
    }
    bActive=0;
    FREEZE->setEnabled(false);
    //exEnableItem(hWnd, IDM_ACTIVE, TRUE);
    //InvalidateRect(hWnd,NULL,FALSE);

    okSetSeqCallback(hBoard, nullptr, nullptr, nullptr); //confirm to null
}

//Active
void MainWindow::OnActiveClicked()
{
    //must not to match client rect
    ui->MATCHCLIENT->setChecked(false);
    bMatchClient=0;

    //must not to upright
    ui->UPTORIGHT->setChecked(false);
    ui->UPTOLEFT->setChecked(false);
    okSetConvertParam(hBoard,CONVERT_UPRIGHT,0); //un to upright


    lScrnOffset=exSetToWndRectEx(hBoard, bScrnCenter);
    //UpdateWindow(hWnd);
    okStopCapture(hBoard); //confirm


    //specail capture to frame
    if( okGetTypeCode(hBoard, nullptr)==OK_C70 ) {
        okCaptureTo(hBoard,FRAME,0,0);
        //exEnableItem(hWnd, IDM_ACTIVE, FALSE);
        FREEZE->setEnabled(true);
    } //normal to screen
    else if( okCaptureToScreen(hBoard)<=0 ) { //not support
        long	errcode=okGetLastError();

        if(errcode==ERR_FORMNOTSUPPORT) {

            char	str[120],str1[120];
            long	bits;

            bits=okSetCaptureParam(hBoard,CAPTURE_SCRRGBFORMAT,-1);
            sprintf(str," Direct Capture To Screen");
            okGetTypeCode(hBoard,str1);
            sprintf(str1+strlen(str1)," does not support current %d bits mode !\n\n"
                "Please try other display mode.",HIWORD(bits));
            QMessageBox::warning(this, str, str1);

            //MessageBox(hwnd,""
            //	"\tLook up which format this card support ? \n\t\t Try 256 color or 24 bits color mode.",
            //	"This card does not support current display mode !",MB_OK);
        }
        else {//
            char	str[100];
            sprintf(str,"Some thing wrong ! Error Code = %d",errcode);
            QMessageBox::warning(this, "Error", str);
        }
    }
    else {  //normal capture alive to screen
        //exEnableItem(hWnd, IDM_ACTIVE, FALSE);
        FREEZE->setEnabled(true);
        bActive=1;
        if (ui->BACKTOSCREEN->isChecked())
            bDispToScreen=1;
        else
            bDispToScreen=0;
//        exCreateWatching(hBoard, hWnd, 0); //to screen
    }
}

//SingleCapTo
void MainWindow::OnSingleCapToClicked()
{
    OnFreezeClicked();
    //bActive=0;
    //exEnableItem(hWnd, IDM_FREEZE, FALSE);
    //exEnableItem(hWnd, IDM_ACTIVE, TRUE);
    //exStopMulCapture(); //to stop in case mul-capturing

    if( okGetTypeCode(hBoard, nullptr)==OK_C70 ) { //to frame
        okCaptureTo(hBoard,FRAME,0,1);
//test						okGetCaptureStatus(hBoard,1);

        if(okConvertRect(hBoard,SCREEN,0,FRAME,0,1)>0) {
            //okConvertRect(hBoard,BUFFER,0,FRAME,0,0x30001); //test
            //take buffer size
            exGetBitmapHeader(hBoard, BUFFER, lpbi);
            exSetDataToDIB(SCREEN, 0, lpbi, lpdib);
        }
    } else { //all other normal grabber device
        long	basetime,elapse,times,times1,times2,lret;
        char	str[120];

        strcpy(str,"");
        lret=okSetCaptureParam(hBoard,CAPTURE_TRIGCAPTURE,-1);
        if( LOWORD(lret)>0 ) {
            sprintf(str," Triggered capture mode!   Now waiting for trigger ...");
        }
        else {
            lret=okSetVideoParam(hBoard,VIDEO_SIGNALTYPE,-1);
            if(HIWORD(lret))
                sprintf(str," Video by external trigger!   Now waiting for video of next frame ...");
        }

        if(strlen(str))
            this->setWindowTitle(str);


        basetime=okGetTickCount();
        //okCaptureTo(hBoard,SCREEN,0,1);//single
        //okCaptureSingle(hBoard,BUFFER,0);//single
        okCaptureTo(hBoard,BUFFER,0,1);//single
        //okCaptureSequence(hBoard,0,1);//single
        elapse=okGetTickCount();
        //okCaptureSequence(hBoard,0,1);//single
        okGetCaptureStatus(hBoard,1);
        Sleep(80);
        times=okGetTickCount();
        //okSleep(20);


        //buf->scr
        exGetBitmapHeader(hBoard, BUFFER, lpbi);
        exSetDataToDIB(BUFFER, 0, lpbi, lpdib);
        times1=okGetTickCount();

        lScrnOffset=exSetToWndRectEx( hBoard, bScrnCenter);
        update();

        times2=okGetTickCount();
        //okStopCapture(hBoard);

        sprintf(str," Start=%d ms,  Capture=%d ms,  Trans=%d,  Paint=%d ms",elapse-basetime,times-elapse,times1-times, times2-times1);
        this->setWindowTitle(str);

    }
}

//help
void MainWindow::OnTestVGACard()
{
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    bActive=1;
    //must not to upright
    ui->UPTORIGHT->setChecked(false);
    ui->UPTOLEFT->setChecked(false);
    okSetConvertParam(hBoard,CONVERT_UPRIGHT,0); //un to upright

    //restore to normal (system palette)
    //okSetConvertParam(hBoard, CONVERT_PALETTE, (LPARAM)NULL);
    //direct to vga memory
    if (ui->BACKTOSCREEN->isChecked())
        okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);

    //okSetToWndRect(hBoard,hWnd);
    lScrnOffset=exSetToWndRectEx(hBoard, bScrnCenter);
    //okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,0);
    okCaptureByBuffer(hBoard,SCREEN,0,0);//0: cont.
    //{
    //	RECT	rect;
    //	GetClientRect(hWnd,&rect);
    //	okSetTargetRect(hBoard,(TARGET)hWnd,&rect);
    //}
    //okCaptureByBuffer(hBoard,(TARGET)hWnd,0,0);//0: cont.
}

void MainWindow::OnTestDrawWnd()
{
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    bActive=-1;
    //must not to upright
    ui->UPTORIGHT->setChecked(false);
    ui->UPTOLEFT->setChecked(false);
    okSetConvertParam(hBoard,CONVERT_UPRIGHT,0); //un to upright

    //okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,2);
    //direct to wnd client
    {
        RECT	rect;
        if(bMatchClient) {
            //
            //GetClientRect(hWnd,&rect);
            //okSetTargetRect(hBoard,(TARGET)hWnd,&rect);
            //auto matching mode:
            rect.left=1;rect.right=0; //to match window client
            rect.top=rect.bottom=0;
            okSetTargetRect(hBoard,(TARGET)winId(), &rect);
        }
        else { //same as set as screen
            //not matching mode: way 1 to take screen rect preset
            //okSetTargetRect(hBoard,(TARGET)hWnd,NULL); //clear rect set

            rect.left=rect.right=0; //not to match window client
            //not matvhing mode: way 2
            //rect.right=-1;
            //okSetTargetRect(hBoard,(TARGET)SCREEN, &rect);
            //MapWindowPoints(HWND_DESKTOP, hWnd, (LPPOINT)&rect,2);

        }
        if (ui->BACKTOSCREEN->isChecked())
            okSetSeqCallback(hBoard,exBeginCapture,exBackDisplay,exEndCapture);
        okCaptureByBuffer(hBoard,(TARGET)winId(),0,0);//0: cont.

    }
}

void MainWindow::OnTestMainWnd()
{
    OnFreezeClicked();
    FREEZE->setEnabled(true);
    bActive=-1;
    if (ui->BACKTOSCREEN->isChecked())
        bDispToScreen=1;
    else
        bDispToScreen=0;

//    hLoopThread=exCreateLoopCapture(hBoard, hWnd);
}

void MainWindow::OnDisableActive(bool bValue)
{
    long	lset;

    lset=okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,-1);
    //if(MF_CHECKED==LOWORD(GetMenuState(hMenu,IDM_DISABLEACTIVE,MF_BYCOMMAND)))
    if(!(lset&0x800000)) { //allow to be change
        lset |= 0x800000;  // set special flag
        if(lset&0x80) //disabled
        {
            lset &= ~0x81;  // enable
            okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,lset);
            DISABLEACTIVE->setChecked(false);
        }
        else {
            lset |= 0x81; //disable and change to control by interrupt
            /*
    {
        char tstr[80];
        sprintf(tstr, "%x  %x", lset, lset|0x81);
    MessageBox(NULL,tstr,NULL,MB_OK);
    }
    */
            okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,lset);
            DISABLEACTIVE->setChecked(true);
        }
        intrMISCCONTROL=okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,-1)&0x3;
    }
}

void MainWindow::OnDisableSyncVert()
{
    long	lset;

    lset=okSetConvertParam(hBoard,CONVERT_NOSYNCSCREEN,-1); //0: sync
    if(lset) //disabled
    {
        okSetConvertParam(hBoard,CONVERT_NOSYNCSCREEN,0); //un-disable
        DISABLESYNCVERT->setChecked(false);
    }
    else {
        okSetConvertParam(hBoard,CONVERT_NOSYNCSCREEN,1);//disable
        DISABLESYNCVERT->setChecked(true);
    }
}

void MainWindow::OnSetAllocBuf()
{
    BufferDlgBox m_dlg;

    if(bActive>0)
        okStopCapture(hBoard);

    m_dlg.exec();
}

void MainWindow::OnLockBufSize()
{
    SetLockBufferBox m_dlg;

    if(bActive>0)
        okStopCapture(hBoard);

    m_dlg.exec();
}


void MainWindow::OnRegDevDriver()
{
    okSetNTDriver(1); //mainly for NT4
}

void MainWindow::OnSignalClicked()
{
    SignalBox m_dlg;

    if(bActive>0)
        okStopCapture(hBoard);

#if defined(Q_OS_WIN32)
    MessageBeep(MB_ICONEXCLAMATION);
#endif

    m_dlg.SetBoard(hBoard);

    m_dlg.exec();
    exAllocateDIBMemory(hBoard);
    //dest(buffer) rect may be changed
    exGetBitmapHeader(hBoard, BUFFER, lpbi);
    //get current size
    sampwidth=(short)lpbi->biWidth;
    sampheight=(short)lpbi->biHeight;
    lScrnOffset=exSetToWndRectEx(hBoard, bScrnCenter);
    update();
}

void MainWindow::OnExitTrigger()
{
    char	string[128];
    int		trigger;


    okCaptureToScreen(hBoard);
    sprintf(string," Waiting extern trigger...");
    setWindowTitle(string);
    trigger=okWaitSignalEvent(hBoard,EVENT_EXTTRIGGER,10000);//max 10 seconds
    if(trigger==1)
        sprintf(string," Extern trigger came.");
    else if(trigger==-1)
        sprintf(string," Not support trigger");
    else
        sprintf(string," Waiting time out");
    okStopCapture(hBoard);
        //SendMessage(hWnd,WM_COMMAND,IDM_FREEZE,0);
    setWindowTitle(string);
}

void MainWindow::OnAboutClicked()
{
    AboutBox m_dlg;

    if(bActive>0)
        okStopCapture(hBoard);

    m_dlg.SetBoard(hBoard);
    m_dlg.SetCurrentNo(iCurrUsedNo);
    m_dlg.SetMaxMem(dwMaxMemSize);

    m_dlg.exec();
}



















//----------------------- private routines----------



//------------------

BOOL MainWindow::InitSetMenu(HANDLE hBoard)
{
    long	typecode;
    char	okname[120];

    intrMISCCONTROL=okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,-1)&0x3;

    //exGetBitmapHeader(hBoard, SCREEN, lpbi);
    exGetBitmapHeader(hBoard, BUFFER, lpbi); //set-lpbi
    //set initial size
    sampwidth=(short)lpbi->biWidth;
    sampheight=(short)lpbi->biHeight;
    //initailize screen
    lScrnOffset= exSetToWndRectEx(hBoard, bScrnCenter);
    update();

    //just test
    okGetBufferAddr(hBoard, 0);
    if( okGetBufferSize( hBoard, nullptr, nullptr) <2 ) {
        QMessageBox::about(this, "Warning", "     Some operation can not run correctly !\n Because BUFFER is not enough for current image size. ");
    }
    {
        long	format=okSetCaptureParam(hBoard,CAPTURE_SCRRGBFORMAT,-1);

        switch(LOWORD(format))
        {
            case 	FORM_RGB565:
            case 	FORM_RGB555:
            //case 	FORM_RGB332:
                QMessageBox::about(this, "Warning", "     Current display mode is 16 bits ! \n\n You should change it to 8, 24 or 32 bits mode");
                break;

        }
    }

    okSetConvertParam(hBoard,CONVERT_UPRIGHT,0); //init to normal
    ui->UPTORIGHT->setChecked(false);
    ui->UPTOLEFT->setChecked(false);

    okSetConvertParam(hBoard,CONVERT_MIRROR,0);//init to normal
    ui->MAKEXMIRROR->setChecked(false);
    ui->MAKEYMIRROR->setChecked(false);

    //modelcode=okGetTypeCode(hBoard,-1); //just get model code
    typecode=okGetTypeCode(hBoard,okname);
    if(typecode<0) //virtual
        strcat(okname,"_");
//    ModifyMenu(hMenu,IDM_OKMODEL,MF_BYCOMMAND|MF_STRING|MF_GRAYED,IDM_OKMODEL,okname);


    if( okSetVideoParam(hBoard,VIDEO_FRAMELINES,-1)!=-1) { //support playback
        ui->PlayBack->setEnabled(true);

        if( (okFillOutLUT(hBoard, nullptr, 0, 0) <=0) || //not suuport outlut
             (typecode==OK_M30) ) {//special for M30 without outlut
            ui->NORMALLUT->setEnabled(false);
            ui->INVERSELUT->setEnabled(false);
            ui->ABSOLUTELUT->setEnabled(false);
            ui->ABSOLUTEDSA->setEnabled(false);
        }
    }
    else {
        ui->PlayBack->setEnabled(false);
    }

    if( okFillInputLUT(hBoard, nullptr, 0, 0)<=0 ) {
        ui->NORMALLUT->setEnabled(false);
        ui->INVERSELUT->setEnabled(false);
    }
    else {
        ui->NORMALLUT->setEnabled(true);
        ui->INVERSELUT->setEnabled(true);
    }

    //
    if( (okFillInputLUT(hBoard, nullptr, 0, 0)>0 ) ||
            (okFillOutLUT(hBoard, nullptr, 0, 0)>0) )    //set in &out lut
        ui->FILLINGLUT->setEnabled(true);
    else
        ui->FILLINGLUT->setEnabled(false);


    if( LOWORD(okSetCaptureParam(hBoard,CAPTURE_FRMRGBFORMAT,-1))!=0xffff ) { //support frame
        ui->CAPTOFRAME->setEnabled(true);
        ui->DISPFROMFRAME->setEnabled(true);
    }
    else {
        ui->CAPTOFRAME->setEnabled(false);
        ui->DISPFROMFRAME->setEnabled(false);
    }

    if( okTransferRect(hBoard,FRAME,0,BUFFER,0,0) >0 )  { //buffer to frame
        ui->BUFFERTOFRAME->setEnabled(true);
    }
    else {
        ui->BUFFERTOFRAME->setEnabled(false);
    }

    if( okTransferRect(hBoard,BUFFER,0,FRAME,0,0) >0 )  { //frame to buffer
        ui->FRAMETOBUFFER->setEnabled(true);
        ui->FRAMETOSCREEN->setEnabled(true);
    }
    else {
        ui->FRAMETOBUFFER->setEnabled(false);
        ui->FRAMETOSCREEN->setEnabled(false);
    }

    if( typecode==OK_M60 ) { //special for M60 with frame
        ui->CAPTOFRAME->setEnabled(true);
        ui->BUFFERTOFRAME->setEnabled(true);
    }
    else if( typecode==OK_RGB30 ) { //special for RGB30 with frame
        ui->CAPTOFRAME->setEnabled(true);
        ui->FRAMETOBUFFER->setEnabled(true);

        ui->FRAMETOSCREEN->setEnabled(true);
    }

    if( okOpenAudio(hBoard, 1) ) { //support audio funcrion
        ui->CAPTUREAUDIO->setEnabled(true);
    }
    else {
        ui->CAPTUREAUDIO->setEnabled(false);
    }

    if( okOpenStream(hBoard, 1) ) { //support stream funcrion
        ui->CAPTUREMPEG2->setEnabled(true);
    }
    else {
        ui->CAPTUREMPEG2->setEnabled(false);
    }


    //reg device driver for nt/2k/xp
    if(  okSetNTDriver(0) ) { // TRUE:registered already, -1:not nt system
        REGDEVDRIVER->setEnabled(false);
    }

    {
    //	short	chan=2;
    //	okLoadInitParam(hBoard,chan);
    }
    //okSaveInitParam(hBoard,2);


    //get current mask mode
    bMaskMode=(short)okEnableMask(hBoard,-1);
    if(bMaskMode==-1) {//not support
        ui->ENABLEMASK->setChecked(false);
        ui->ENABLEMASK->setEnabled(false);
    }
    else {
        ui->ENABLEMASK->setEnabled(true);
        if(bMaskMode)//
            ui->ENABLEMASK->setChecked(true);
        else
            ui->ENABLEMASK->setChecked(false);

        exDemoMask(hBoard, bMaskMode);
    }

    if( okCaptureSequence(hBoard, 0, 0)==-1)
    {//interrupt to capture
        ui->SEQCAPTOBUF->setEnabled(false);
        ui->LOOPSEQCAPTOBUF->setEnabled(false);
        ui->INTERTOINDIRECT->setEnabled(false);
    }
    else {
        ui->SEQCAPTOBUF->setEnabled(true);
        ui->LOOPSEQCAPTOBUF->setEnabled(true);
        ui->INTERTOINDIRECT->setEnabled(true);
    }



    //disable active and display sync with vert
    {
        long	lset;

        lset=okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,-1);
        //if(MF_CHECKED==LOWORD(GetMenuState(hMenu,IDM_DISABLEACTIVE,MF_BYCOMMAND)))
        if((lset&0x800000) ||(lset&0x80)) //disabled
        {
            DISABLEACTIVE->setChecked(true);
        }
        else {
            DISABLEACTIVE->setChecked(false);
        }


        lset=okSetConvertParam(hBoard,CONVERT_NOSYNCSCREEN,-1); //0: sync
        if(lset) //disabled
        {
            DISABLESYNCVERT->setChecked(true);
        }
        else {
            DISABLESYNCVERT->setChecked(false);
        }

    }



    return 1;
}

BOOL MainWindow::exM2VSaveName(char *szFileName)
{
    BOOL    bRet = TRUE;

    QString qsFileName;
    QStringList szFilter;
    QFileDialog* fd = new QFileDialog(this);
    szFilter << "MPEG-2 Video/Audio file (*.M2V)"
             << "MPEG-2 Video/Audio file (*.MPG)"
             << "Compressed AVI in MJPEG  (*.AVI)"
             << "Compressed AVI in MPEG4  (*.AVI)";
    //设置文件过滤器
    fd->setNameFilters(szFilter);
    fd->setAcceptMode(QFileDialog::AcceptSave);
    if ( fd->exec() == QDialog::Accepted )   //如果成功的执行
    {
        qsFileName = fd->selectedFiles()[0];            //取第一个文件名

        if ((qsFileName.right(4) == ".seq") ||
                (qsFileName.right(4) == ".avi") ||
                (qsFileName.right(4) == ".m2v") ||
                (qsFileName.right(4) == ".mpg"))
        {
            QFile file(qsFileName);

            //first delete ok.seq or .avi
            file.remove();
        }
        QTextCodec::setCodecForLocale(QTextCodec::codecForName("GBK"));
        QByteArray ba = qsFileName.toLocal8Bit();
        strcpy(szFileName, ba.data());

        int i, total;

        total = szFilter.size();

        for (i=0; i<total; i++)
            if (szFilter.at(i) == fd->selectedNameFilter())
            {
                if(i==3) { //avi seq file in jpeg data
                    strcat(szFileName,",mjpg"); //avi in mjpg
                }
                else if(i==4) { //avi seq file in mpg4 data
                    strcat(szFileName,",mpg4"); //avi in mpg4
                }
            }
    }
    else
    {
        bRet = FALSE;
    }

    fd->close();
    delete fd;

    return bRet;
}

BOOL MainWindow::exGetSaveName(char *szFileName)
{
    BOOL    bRet = TRUE;
    int     iFilterIndex = 0;

    QString qsFileName;
    QStringList szFilter;
    QFileDialog* fd = new QFileDialog(this);
    szFilter << "BMP file (*.BMP)"
             << "Jpeg format file (*.JPG)"
             << "Sequence file (*.SEQ)"
             << "Sequence file in JPEG (*.SEQ)"
             << "Audio-video interleaved (*.AVI)"
             << "Compressed AVI in MJPEG (*.AVI)"
            #ifndef Q_PROCESSOR_X86_64
            //这是32位应用
                << "Compressed AVI in MPEG4 (*.AVI)"
            #else
            //这是64位应用
                << "Compressed AVI in H265 (*.AVI)"
            #endif
             << "Compressed AVI in H264 (*.AVI)"
             << "Compressed in H264 (*.MP4)"
            #ifndef Q_PROCESSOR_X86_64
                << "MPEG2(DVD) file (*.mpg)"
            #else
                << "TS format file (*.ts)"
            #endif
             << "Jpeg 2000 format file (*.JP2)"
             << "TIFF format file (*.TIF)"
             << "Raw format files (*.RAW)";
    //设置文件过滤器
    fd->setNameFilters(szFilter);
    fd->setAcceptMode(QFileDialog::AcceptSave);
    if ( fd->exec() == QDialog::Accepted )   //如果成功的执行
    {
        int i, total;

        total = szFilter.size();

        for (i=0; i<total; i++)
        {
            if (szFilter.at(i) == fd->selectedNameFilter())
            {
                iFilterIndex = i;
                break;
            }
        }

        switch (iFilterIndex)
        {
        case 0:
            fd->setDefaultSuffix("bmp");
            break;
        case 1:
            fd->setDefaultSuffix("jpg");
            break;
        case 2:
            fd->setDefaultSuffix("seq");
            break;
        case 3:
            fd->setDefaultSuffix("seq");
            break;
        case 4:
            fd->setDefaultSuffix("avi");
            break;
        case 5:
            fd->setDefaultSuffix("avi");
            break;
        case 6:
            fd->setDefaultSuffix("avi");
            break;
        case 7:
            fd->setDefaultSuffix("avi");
            break;
        case 8:
            fd->setDefaultSuffix("mp4");
            break;
        case 9:
            fd->setDefaultSuffix("jp2");
            break;
        case 10:
            #ifdef _WIN64
                fd->setDefaultSuffix("ts");
            #else
                fd->setDefaultSuffix("mpg");
            #endif
            break;
        case 11:
            fd->setDefaultSuffix("tif");
            break;
        case 12:
            fd->setDefaultSuffix("raw");
            break;
        default:
            fd->setDefaultSuffix("seq");
            break;
        }

        qsFileName = fd->selectedFiles()[0];            //取第一个文件名

        if ((qsFileName.right(4) == ".seq") ||
                (qsFileName.right(4) == ".avi") ||
                (qsFileName.right(4) == ".m2v") ||
                (qsFileName.right(4) == ".mpg")||
                (qsFileName.right(4) == ".mp4")||
                (qsFileName.right(4) == ".ts"))
        {
            QFile file(qsFileName);

            //first delete ok.seq or .avi...
            file.remove();
        }

        QTextCodec::setCodecForLocale(QTextCodec::codecForName("GBK"));
        QByteArray ba = qsFileName.toLocal8Bit();
        strcpy(szFileName, ba.data());

        if (iFilterIndex==3) { //seq file in jpeg data
            strcat(szFileName,",jpg,50"); //quality=50
        }
        else if (iFilterIndex==5) { //avi in mjpg
            strcat(szFileName,",mjpg,60"); //mjpeg
        }
        else if (iFilterIndex==6) { //avi in mpeg4
            #ifdef _WIN64
                strcat(szFileName,",H265");
            #else
                strcat(szFileName,",mpg4"); //mpeg4 ,default 6 (2~31)//x86
            #endif

        }
        else if (iFilterIndex==7) { //avi in h264
            strcat(szFileName,",h264"); //mpeg4 ,default 6 (2~31)
        }
    }
    else
    {
        bRet = FALSE;
    }
    fd->close();
    delete fd;

    return bRet;
}

BOOL MainWindow::exGetLoadName(char *szFileName)
{
    BOOL    bRet = TRUE;

    QString qsFileName;
    QStringList szFilter;
    QFileDialog* fd = new QFileDialog(this);
    szFilter << "BMP file (*.BMP)"
             << "Jpeg format file (*.JPG,JPEG)"
             << "Sequence file (*.SEQ)"
             << "Audio-video interleaved (*.AVI)"
             << "Mpeg2 Video stream (*.M2V)"
             << "Mpeg2 Video stream (*.MPG)"
             << "Jpeg 2000 format file (*.JP2)"
             << "TIFF file  (*.TIF)"
             << "GIF file  (*.GIF)"
             << "Raw format files (*.RAW)"
             << "All Above format files(*.BMP;*.JPG;*.JPEG;*.SEQ;*.AVI;*.M2V;*.MPG;*.JP2;*.TIF;*.GIF;*.RAW)";


    //设置文件过滤器
    fd->setNameFilters(szFilter);
    fd->setAcceptMode(QFileDialog::AcceptOpen);
    if ( fd->exec() == QDialog::Accepted )   //如果成功的执行
    {
        qsFileName = fd->selectedFiles()[0];            //取第一个文件名
        QTextCodec::setCodecForLocale(QTextCodec::codecForName("GBK"));
        QByteArray ba = qsFileName.toLocal8Bit();
        strcpy(szFileName, ba.data());
    }
    else
    {
        bRet = FALSE;
    }

    fd->close();
    delete fd;

    return bRet;
}

BOOL MainWindow::exLoadImageFile(HANDLE hBoard)
{
    char		szFileName[512]="\0";
    long		total;
    long		length=1;
    BLOCKINFO	blkinfo;


    if(!exGetLoadName(szFileName))
        return FALSE;

    if( !strrchr(szFileName,'.') )
        return FALSE;

    memset(&blkinfo,0,sizeof(BLOCKINFO)); //zero all for init

    if( !strnicmp(strrchr(szFileName,'.'),".seq",4) ||
            !strnicmp(strrchr(szFileName,'.'),".avi",4) ||
                !strnicmp(strrchr(szFileName,'.'),".m2v",4) ||
                    !strnicmp(strrchr(szFileName,'.'),".mpg",4)
            ) {
        //first get inf about seq file
        total=okLoadImageFile(hBoard,szFileName,0,(TARGET)&blkinfo,0,0);//no=0

        //how many
        SetSeqNum m_SSNDlg;
        m_SSNDlg.SetNum(total);
        length=total;
        if (m_SSNDlg.exec() == QDialog::Accepted)
        {
            total = m_SSNDlg.GetNum();

            //EditMJPGAviFile(hBoard, szFileName); //test

            //then load them into buffer
            if( length=okLoadImageFile(hBoard,szFileName,0,BUFFER,0,total) ) {
                //last set first block to dib
                EXTRAFRMINFO	extraInfo; //


                //buffer size changed
                if( !exAllocateDIBMemory(hBoard) ) {
                        return 0;
                }

                okGetCurrImageInfo(hBoard, (LPARAM)&extraInfo, sizeof(EXTRAFRMINFO));

                exGetBitmapHeader(hBoard, BUFFER, lpbi);
                exSetDataToDIB(BUFFER, 0, lpbi, lpdib);
            }

        }
    }
    else { //if( !stricmp(strrchr(szFileName,'.'),".bmp") ){//bmp
        //load bmp or raw into buffer and dib
        if(okGetBufferSize(hBoard,NULL,(MWORD *)&dwBufSize) ) { //buffer being
            long	lParam1=0, lNum=1;
            RAWFILEPARAM sRawFile;

            if( !stricmp(strrchr(szFileName,'.'),".raw") ){
                memset(&sRawFile,0,sizeof(RAWFILEPARAM)); //zero all for init
                lParam1=(MLONG)(&sRawFile);
            }

            //memset(&blkinfo,0,sizeof(BLOCKINFO)); //zero all for init

            blkinfo.lpBits = nullptr; //only get block info
            if(okLoadImageFile(hBoard,szFileName,lParam1,(TARGET)&blkinfo,0,1)<0)
                return 0;
            if(sRawFile.wPixBits) //filled
                sRawFile.wRawFlag=RAWFILE;

            //RAWFILEPARAM RawFile;
            //if( !stricmp(strrchr(szFileName,'.'),".raw") ){
            //	RawFile.wRawFlag=RAWFILE;
            //	RawFile.wPixBits=8;
            //	RawFile.dwWidth=576;
            //	RawFile.dwHeight=768;
            //	RawFile.dwOffset=0;
            //	lParam1=(long)(&RawFile);
            //}
            //okLoadImageFile(hBoard,szFileName,0,SCREEN,0,1); //test


            //how many
            SetSeqNum m_SSNDlg;
            m_SSNDlg.SetNum(lNum);
            if (m_SSNDlg.exec() == QDialog::Accepted)
            {
                lNum = m_SSNDlg.GetNum();

                //then load them into buffer
                if( length=okLoadImageFile(hBoard,szFileName,lParam1,BUFFER,0,lNum) ) {
                    //buffer size changed
                    if( !exAllocateDIBMemory(hBoard) ) {
                            return 0;
                    }

                    //last set first block to dib
                    exGetBitmapHeader(hBoard, BUFFER, lpbi);
                    exSetDataToDIB(BUFFER, 0, lpbi, lpdib);
                }
            }
        }
        else if(hBoard) { // no buffer
            //way 2
            //just load to dib (& screen)
            memset(&blkinfo,0,sizeof(BLOCKINFO)); //zero all for init

            blkinfo.lpBits = nullptr; //only get block info
            okLoadImageFile(hBoard,szFileName,0,(TARGET)&blkinfo,0,1);

            // set dib info
            exGetBitmapHeader(hBoard, (TARGET)&blkinfo, lpbi);
            //buffer size changed
            if( !exAllocateDIBMemory(hBoard) ) {
                return 0;
            }

            //get dib for wnd paint
            blkinfo.lpBits=lpdib;
            blkinfo.lpExtra=(LPBYTE)lpbi+lpbi->biSize;  //do here
            if( length=okLoadImageFile(hBoard,szFileName,0,(TARGET)&blkinfo,0,1) ){
                exConvertBitmap(lpbi,(LPSTR)lpdib); //invert y
            }

            //okLoadImageFile(hBoard,szFileName,0,SCREEN,0,1);
        }
    }

    if(length==0)
        QMessageBox::warning(this, "Warning !", " Load file failed ?!  Try to check the file or memory size. ");

    return TRUE;
}

BOOL MainWindow::exSaveScreenFile(HANDLE hBoard)
{
    char		szFileName[512]="\0";
    BLOCKINFO	blkinfo;
    long		lret;

    if( !exGetSaveName(szFileName) )
        return FALSE;

    if( !strrchr(szFileName,'.') )
        return FALSE;

    //save current dib
    ZeroMemory(&blkinfo,sizeof(BLOCKINFO)); //do zero
    blkinfo.iWidth=(short)lpbi->biWidth;
    blkinfo.iHeight=(short)lpbi->biHeight;
    blkinfo.iBitCount=(short)lpbi->biBitCount;
    blkinfo.lpBits=lpdib;
    blkinfo.lpExtra=(LPBYTE)lpbi+lpbi->biSize;

    blkinfo.iFormType=0; //unknown
    //special for 565
    if(lpbi->biBitCount==16) {
        DWORD	*lpMask=(DWORD *)blkinfo.lpExtra;

        if( lpMask[1]==0x07e0) //565
            blkinfo.iFormType=FORM_RGB565;
        else
            blkinfo.iFormType=FORM_RGB555;
    }

    //first inverse dib to bitmap
    exConvertBitmap(lpbi, (LPSTR)lpdib);
    lret=okSaveImageFile(hBoard,szFileName,0,(TARGET)&blkinfo,0,1);
    //restore to dib
    exConvertBitmap(lpbi, (LPSTR)lpdib);

    if(lret==0)
        QMessageBox::warning(this, "Warning !", " Can't save file ?! Try to check media to store. ");

    return TRUE;
}

BOOL MainWindow::exSaveBufferFile(HANDLE hBoard)
{
    char		szFileName[512]="\0";
    BLOCKINFO	blkinfo;
    long		elpased;
    char		string[100];
    long		lret = 0;


    ZeroMemory(&blkinfo,sizeof(BLOCKINFO)); //do zero

    if( !exGetSaveName(szFileName) )
        return FALSE;

    if( !strrchr(szFileName,'.') )
        return FALSE;

    if( !strnicmp(strrchr(szFileName,'.'),".seq",4) ||
            !strnicmp(strrchr(szFileName,'.'),".avi",4) ||
            !strnicmp(strrchr(szFileName,'.'),".m2v",4) ||
            !strnicmp(strrchr(szFileName,'.'),".mpg",4)
                ) {
        long		total;

        //save buffer seq
        total=(short)okGetBufferSize(hBoard, nullptr, nullptr);
        //how many
        SetSeqNum m_SSNDlg;
        m_SSNDlg.SetNum(total);
        if (m_SSNDlg.exec() == QDialog::Accepted)
        {
            //BLOCKINFO	blk;
            //ZeroMemory(&blk,sizeof(BLOCKINFO));
            //blk.iFormType=FORM_GRAY10;
            //blk.iBitCount=10;
            //blk.iWidth=768;
            //blk.iHeight=576;
            //blk.lpBits=okGetBufferAddr(hBoard,0);

            total = m_SSNDlg.GetNum();

            elpased=okGetTickCount();

            lret = okSaveImageFile(hBoard,szFileName,0,BUFFER,0,total);	//BUFFER

            elpased=okGetTickCount()-elpased;

            //than set this first block to dib to see what we saved
            exGetBitmapHeader(hBoard, BUFFER, lpbi);
            exSetDataToDIB(BUFFER, 0, lpbi, lpdib);
        }
    }
    else { //bmp or other single
        if(okGetBufferSize(hBoard, nullptr, (MWORD *)&dwBufSize) ) { //buffer being
            //how many
            SetSeqNum m_SSNDlg;
            m_SSNDlg.SetNum(1);
            if (m_SSNDlg.exec() == QDialog::Accepted)
            {

                elpased=okGetTickCount();

                lret=okSaveImageFile(hBoard,szFileName,0,BUFFER,0,m_SSNDlg.GetNum());

/*
                //blkinfo.iWidth=(short)lpbi->biWidth;
            //blkinfo.iHeight=(short)lpbi->biHeight;
            blkinfo.lBlockStep=0;
            blkinfo.iHiStep=0;
            blkinfo.iBitCount=(short)lpbi->biBitCount;
            blkinfo.lpBits=okGetBufferAddr(hBoard,3);;
            //blkinfo.lpExtra=(LPSTR)lpbi+lpbi->biSize;
            blkinfo.iType=BLKHEADER;//BUFHEADER; //unknown
            blkinfo.iFormType=FORM_RGB888;
            blkinfo.lTotal=3;
            okCaptureByBuffer(hBoard,(TARGET)&blkinfo,0,3);

            Sleep(200);
            //okStopCapture(hBoard);
            Sleep(200);

            okSaveImageFile(hBoard,szFileName,0,(TARGET)&blkinfo,0,3);
*/

                elpased=okGetTickCount()-elpased;

                //then set this first block to dib to see what we saved
                exGetBitmapHeader(hBoard, BUFFER, lpbi);
                exSetDataToDIB(BUFFER, 0, lpbi, lpdib);
            }
        }
        else { //no buffer allocated
            //save current dib
            blkinfo.iWidth=(short)lpbi->biWidth;
            blkinfo.iHeight=(short)lpbi->biHeight;
            blkinfo.lBlockStep=0;
            blkinfo.iHiStep=0;

            blkinfo.iBitCount=(short)lpbi->biBitCount;
            blkinfo.lpBits=lpdib;
            blkinfo.lpExtra=(LPBYTE)lpbi+lpbi->biSize;

            blkinfo.iFormType=0; //unknown


            //special for 565
            if(lpbi->biBitCount==16) {
                DWORD	*lpMask=(DWORD *)blkinfo.lpExtra;

                if( lpMask[1]==0x07e0) //565
                    blkinfo.iFormType=FORM_RGB565;
                else
                    blkinfo.iFormType=FORM_RGB555;
            }

            //first inverse dib to bitmap
            exConvertBitmap(lpbi,(LPSTR)lpdib);

            elpased=okGetTickCount();
            lret=okSaveImageFile(hBoard,szFileName,0,(TARGET)&blkinfo,0,1);
            elpased=okGetTickCount()-elpased;

            //restore to dib
            exConvertBitmap(lpbi,(LPSTR)lpdib);
        }
    }
    sprintf(string," Spent times: %d ms ",elpased);
#if defined(Q_OS_WIN32)
    SetWindowTextA((HWND)winId(), string);
#endif

    if(lret==0)
        QMessageBox::warning(this, "Warning !", " Can't save file ?! Try to check media to store. ");

    return TRUE;
}

BOOL MainWindow::exLoadCfgFile(HANDLE hBoard)
{
    QString qsFileName;
    QFileDialog* fd = new QFileDialog(this);
//    fd->resize(240,320);    //设置显示的大小
    fd->setFileMode(QFileDialog::ExistingFile);
    fd->setNameFilter(tr("Config file with full params. (*.OKF);;Config file just for sample (*.OKS);;Config file just for output (*.OKO);;CamLink Mode Config File (*.CLF);;IO Config File (*.IOF)")); //设置文件过滤器
//  fd->setViewMode(QFileDialog::List); 设置浏览模式，有 列表（list） 模式和 详细信息（detail）两种方式
    if ( fd->exec() == QDialog::Accepted)   //如果成功的执行
    {
        qsFileName = fd->selectedFiles()[0];            //取第一个文件名
        QTextCodec::setCodecForLocale(QTextCodec::codecForName("GBK"));
        QByteArray ba = qsFileName.toLocal8Bit();
        okLoadConfigFile(hBoard, ba.data());
    }
    else
    fd->close();

    delete fd;


    return TRUE;
}


//--bitmap.c
BOOL MainWindow::exGetBitmapHeader(HANDLE hBoard, TARGET src, LPBITMAPINFOHEADER lpbi)
{
    short		width,height;
    long		form,extend;


    //take buffer's size and screen' bitcount
    form=exGetTargetSize(hBoard, src, &width, &height);
    extend=okSetCaptureParam(hBoard,CAPTURE_SAMPLEFIELD,-1);
    if(extend==0) { //in field
        extend=okSetConvertParam(hBoard,CONVERT_FIELDEXTEND,-1);
        //if(extend==FIELD_COPYEXTEND || extend==FIELD_COPYINTERPOL)
        //	height*=2;
    }

    if(src<=1) {
        //here take form as screen forever
        form=okSetCaptureParam(hBoard,CAPTURE_SCRRGBFORMAT,GETCURRPARAM); //-1
        //if(HIWORD(form)>8) //take color rgb888	//discarded, re-changed on 1/06/06
        //	form= MAKELONG(FORM_RGB888,24);
        if((form<0)||(form ==0xffff))
        {

            form = 0x200004;
        }
    }

    { //treat rotate
        short temp;

        temp=(short)okSetConvertParam(hBoard,CONVERT_UPRIGHT,-1); //un
        if(temp>0) {
            temp=width;
            width=height;
            height=temp;
        }
    }
    exSetBitmapHeader(lpbi, width, height, HIWORD(form), LOWORD(form) );

    return LOWORD(form);
}

BOOL MainWindow::exMoveToClipboard( LPBITMAPINFOHEADER lpbi, LPBYTE lpdib )
{
    HANDLE	hDIB;
    DWORD	size;
    LPBYTE	lpbuf;

#if defined(Q_OS_WIN32)
    if( OpenClipboard(nullptr) ) {
        size= (lpbi->biClrUsed==256) ? 1024:0;
        if(lpbi->biBitCount==16 || lpbi->biBitCount==32)
            size+=3*sizeof(DWORD);
        size+=lpbi->biSize+lpbi->biSizeImage;

        hDIB=GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE, size);
        lpbuf=(LPBYTE)GlobalLock(hDIB);
        //CopyMemory(lpbuf, lpbi, size-lpbi->biSizeImage);
        //CopyMemory(lpbuf+size-lpbi->biSizeImage, lpdib, lpbi->biSizeImage);
        CopyMemory(lpbuf, lpbi, lpbi->biSize);
        CopyMemory(lpbuf+lpbi->biSize, lpdib, lpbi->biSizeImage);
        GlobalUnlock(hDIB);

        SetClipboardData(CF_DIB,hDIB);
        CloseClipboard();
        return TRUE;
    }
#endif

    return FALSE;
}

BOOL MainWindow::exConvertBitmap(LPBITMAPINFOHEADER lpbi,LPSTR lpdib)
{
    long	i,wbytes;
    LPSTR	lpbuf;
    LPSTR	lptop,lpbottom;

    if(!lpbi)
        return FALSE;
    if(!lpbi->biHeight)
        return FALSE;
    //inverse dib (top to bottom)
    wbytes=lpbi->biSizeImage/lpbi->biHeight;
    lpbuf=(LPSTR)GlobalAlloc(GPTR,wbytes);

    lptop=lpdib;
    lpbottom=lpdib+(lpbi->biHeight-1)*wbytes;
    for(i=0;i<(lpbi->biHeight/2);i++) {
        CopyMemory(lpbuf,lptop,wbytes);
        CopyMemory(lptop,lpbottom,wbytes);
        CopyMemory(lpbottom,lpbuf,wbytes);
        lptop+=wbytes;
        lpbottom-=wbytes;
    }
    GlobalFree(lpbuf);

    return TRUE;
}


//--callback.c
int	MainWindow::exStopMulCapture()
{
    int		i;

    if(NoCapture) {
        for(i=0;i<NoCapture;i++) {
            if(lphBoard[i])
                okStopCapture(lphBoard[i]);
        }

        okUnlockAllBuffer();

        for(i=0;i<NoCapture;i++) {
            //close specified board
            if(lphBoard[i])
                okCloseBoard(lphBoard[i]);
            lphBoard[i] = nullptr;
        }
    }

    NoCapture=0; //clear

    return 1;
}


//--extra.c
//--demo to mask some areas just for those cards with masking capablilty
BOOL MainWindow::exDemoMask(HANDLE hBoard, BOOL bMaskMode)
{
    int			i;
    RECT		rect;
    BYTE		lpMask[40960];
    BLOCKINFO	blk;
#if defined(Q_OS_WIN32)
    LOGFONT 	lfLogFont={ -20, 0, 0, 0, 400, 0, 0, 0, 0, 1, 2, 1, 26, L"Arial"};
#endif
    SETTEXTMODE	textmode={0x0,0xffffff,2,0,0};  //


    //1.------ overlay graph--------------------------------------------------------
    { //set graph overlay
        RECT		rect= {0,0,0,0};
        //SETTEXTMODE	textmode={0xffffff,0x0,COPYFONT,0,0};  //
        SETTEXTMODE	textmode={0xffffff,0x0,FULLCOPY,0,0};  //
        POINT		pt0={100,20},pt1={200,120};
        long		oldGraph=okSetVideoParam(hBoard, VIDEO_ENABLEGRAPHS, -1);
        LPBYTE		lpGraph;
        short		wid,ht;
        MLONG		stride;
        RECT		rcRect;


        //SetRect(&rcRect,8,0,768+8,576);


        //-------capture graph overlay---------------------------------------------------

        rcRect.right=-1;
        okSetTargetRect(hBoard,BUFFER,&rcRect);

        rcRect.left-=100;rcRect.top-=100;
        rcRect.right+=100;rcRect.bottom+=100;

        //okSetVideoParam(hBoard, VIDEO_ENABLEGRAPHS, 0x803);
        //select cap-graph frm0
        okSetVideoParam(hBoard, VIDEO_ENABLEGRAPHS, 0x003);
        lpGraph=(LPBYTE)okGetTargetInfo(hBoard, GRAPH, 0, &wid, &ht, &stride);
        if(lpGraph) //clear
            ZeroMemory(lpGraph, stride*ht);// ZeroMemory


        //okFillCircleTo(hBoard, (TARGET)BUFFER, 0, &rcRect, 1, 0); //graph gray==1 for overlay 0

#if defined(Q_OS_WIN32)
        okFillCircleTo(hBoard, (TARGET)GRAPH, 0, &rcRect, 1, 0);
        okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, (LPSTR)" JOINHOPE嘉恒中自", 16);
        //okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, "H", 1);
        okDrawLineTo(hBoard,(TARGET)GRAPH,0,pt0,pt1,0xff);
        SetRect(&rect,200,120,200,120);
        okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, (LPSTR)" 嘉恒中自JOINHOPE", 16);
#endif


        //select cap-graph frm1
        okSetVideoParam(hBoard, VIDEO_ENABLEGRAPHS, 0x203);
        if(lpGraph) //clear
            ZeroMemory(lpGraph, stride*ht);// ZeroMemory

#if defined(Q_OS_WIN32)
        okFillCircleTo(hBoard, (TARGET)GRAPH, 0, &rcRect, 1, 0);
        SetRect(&rect,0,0,0,0);
        okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, (LPSTR)" JOINHOPE嘉恒中自", 16);
        //okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, "H", 1);
        okDrawLineTo(hBoard,(TARGET)GRAPH,0,pt0,pt1,0xff);
        SetRect(&rect,200,120,200,120);
        okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, (LPSTR)" 嘉恒中自JOINHOPE", 16);
#endif


        //-------output graph overlay---------------------------------------------------
        //SetRect(&rcRect,0,0,768,576);

        //select out-graph frm0
        okSetVideoParam(hBoard, VIDEO_ENABLEGRAPHS, 0x103);
        if(lpGraph) //clear
            ZeroMemory(lpGraph, stride*ht);// ZeroMemory

#if defined(Q_OS_WIN32)
        okFillCircleTo(hBoard, (TARGET)GRAPH, 0, &rcRect, 1, 0);
        SetRect(&rect,0,0,0,0);
        okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, (LPSTR)" JOINHOPE嘉恒中自", 16);
        //okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, "H", 1);
        okDrawLineTo(hBoard,(TARGET)GRAPH,0,pt0,pt1,0xff);
        SetRect(&rect,200,120,200,120);
        okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, (LPSTR)" 嘉恒中自JOINHOPE", 16);
#endif


        //select out-graph frm1
        okSetVideoParam(hBoard, VIDEO_ENABLEGRAPHS, 0x303);
        if(lpGraph) //clear
            ZeroMemory(lpGraph, stride*ht);// ZeroMemory

#if defined(Q_OS_WIN32)
        okFillCircleTo(hBoard, (TARGET)GRAPH, 0, &rcRect, 1, 0);
        SetRect(&rect,0,0,0,0);
        okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, (LPSTR)" JOINHOPE嘉恒中自", 16);
        //okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, "H", 1);
        okDrawLineTo(hBoard,(TARGET)GRAPH,0,pt0,pt1,0xff);
        SetRect(&rect,200,120,200,120);
        okSetTextTo(hBoard, (TARGET)GRAPH, &rect, &lfLogFont, &textmode, (LPSTR)" 嘉恒中自JOINHOPE", 16);
#endif


        //okSetVideoParam(hBoard, VIDEO_ENABLEGRAPHS, 0x003); //enable both graph overlay
        okSetVideoParam(hBoard, VIDEO_ENABLEGRAPHS, oldGraph); //restore

    }




    //2.------ mask graph--------------------------------------------------------
    //1. way (to set whole rect to enable/disable)
    //graph 1.
    rect.left=200;
    rect.top=100;
    rect.right=280;
    rect.bottom=180;
    okSetMaskRect(hBoard, &rect, nullptr); //disable Big Rect video
    InflateRect(&rect,-20,-20);
    okSetMaskRect(hBoard, &rect, LPBYTE(1)); //enable Small Rect video

    //graph 2.
    SetRect(&rect,0,200,200,201);
    okSetMaskRect(hBoard, &rect, nullptr);  //D ---
    SetRect(&rect,0,202,200,203);
    okSetMaskRect(hBoard, &rect, nullptr);  //D ---
    SetRect(&rect,0,205,200,206);
    okSetMaskRect(hBoard, &rect, nullptr);  //D ---



    //2. way (to set mask rect with each pixel setting)

    //graph 3.
    ZeroMemory(lpMask,20000); //clear bits
    for(i=20;i<80;i++)
        lpMask[100+i]=1;		//E ~~~

    for(i=1;i<99;i++)
        lpMask[100*50+i]=1;		//E ---
    for(i=1;i<99;i++)
        lpMask[i*100+50]=1;		//E  |

    for(i=0;i<=99;i++) {
        lpMask[i*100+i]=1;		//E  /
    }
    SetRect(&rect,10,310,110,410); //Disable Rect with + \ ~
    okSetMaskRect(hBoard,&rect,lpMask);

    //graph 4.
    FillMemory(lpMask,40000,1); //set bits
    for(i=1;i<99;i++)
        lpMask[5000+i]=0;		//D ---
    for(i=1;i<99;i++)
        lpMask[i*100+50]=0;		//D  |
    SetRect(&rect,200,319,300,419);	 //disable +
    okSetMaskRect(hBoard,&rect,lpMask);


    //example to set char
    {
        RECT		rect= {0,0,0,0};


        FillMemory(lpMask,40000,1); //set bits to init
        blk.lpBits=lpMask;
        blk.iBitCount=8;
        blk.iWidth=100;
        blk.iHeight=30;
#if defined(Q_OS_WIN32)
        okSetTextTo(hBoard, (TARGET)&blk, &rect, &lfLogFont, &textmode, (LPSTR)"嘉恒中自", 8);
#endif

        SetRect(&rect,300,200,300+100,200+40);	 //set characters
        okSetMaskRect(hBoard,&rect,lpMask);


        FillMemory(lpMask,40000,1); //set bits to init
        SetRect(&rect,0,0,0,0);	 //set characters
#if defined(Q_OS_WIN32)
        okSetTextTo(hBoard, (TARGET)&blk, &rect, &lfLogFont, &textmode, (LPSTR)"JoinHope", 8);
#endif
        SetRect(&rect,300,240,300+100,240+40);	 //set chinese characters
        okSetMaskRect(hBoard,&rect,lpMask);


        /*
        {
        HFONT		hFont;
        extern		HWND	hwnd;
        HDC			hDC;

        hFont=CreateFontIndirect(&lfLogFont);
        hDC=GetDC(hwnd);
        SelectObject(hDC,hFont);

        SetBkColor(hDC,0x0);
        SetTextColor(hDC,0xffff);
        TextOut(hDC,rect.left,rect.top,"JoinHope", 8);
        ReleaseDC(hwnd,hDC);
        DeleteObject(hFont);
        }
        */
    }

    //example to set char and graphics by way of windows hdc
    {
        HANDLE	hDCBitmap;
        HDC		hdcMem;
        RECT	rect= {0,0,200,100};

        FillMemory(lpMask,40000,1); //set bits to init
        blk.lpBits=lpMask;
        blk.iBitCount=8;
        blk.iWidth=200;
        blk.iHeight=100;

#if (0)
        hdcMem=okCreateDCBitmap( hBoard, (TARGET)&blk, &hDCBitmap);

        SetTextColor(hdcMem,0xffffff); //cf
        SetBkColor(hdcMem,0); //cf
        TextOut(hdcMem, 100, 0, L"7123456",7);
        MoveToEx(hdcMem,100,100,NULL);
        LineTo(hdcMem,200,500);
        Ellipse(hdcMem,0,0,200,100);

        okMapDCBitmapTo(hDCBitmap,0);

        okFreeDCBitmap(hDCBitmap);
#endif

        okSetMaskRect(hBoard,&rect,lpMask);

    }

    //turn on mask
    okEnableMask(hBoard,bMaskMode); //1 positive mask
    //okEnableMask(hBoard,2); //2 negtive mode

    return 1;
}

BOOL MainWindow::exFillOutLUT(HANDLE hBoard, short index)
{
    short	i,bits,start=0;


    //check how much bits LUT suppored
    bits=(short)okFillOutLUT(hBoard, nullptr, 0, 0);

    if(	bits<=8) //for 8 bits lut
    {
        BYTE	bLUT[768][3];
        short	iLUT[256],num=255,gray=256;

        switch (index) {
            case 0: //normal
                for(i=0;i<gray;i++) {
                    bLUT[i][0]=(BYTE)i;
                    bLUT[i][1]=(BYTE)i;
                    bLUT[i][2]=(BYTE)i;
                }
                break;
            case 1: //inverse
                for(i=0;i<=num;i++) {
                    bLUT[i][0]=(BYTE)(num-i);
                    bLUT[i][1]=(BYTE)(num-i);
                    bLUT[i][2]=(BYTE)(num-i);
                }
                for(i=num;i<gray;i++) {
                    bLUT[i][0]=0;
                    bLUT[i][1]=0;
                    bLUT[i][2]=0;
                }
                break;
            case 2: //absolute for M60A
                for(i=0;i<128;i++) {
                    bLUT[i][0]=(BYTE)(i);
                    bLUT[i][1]=(BYTE)(i);
                    bLUT[i][2]=(BYTE)(i);
                }
                for(i=128;i<gray;i++) {
                    bLUT[i][0]=(BYTE)(256-i);
                    bLUT[i][1]=(BYTE)(256-i);
                    bLUT[i][2]=(BYTE)(256-i);
                }
                break;

            case 3: //dsa absolute for M60A
                //1. take true value with minus
                for(i=0;i<128;i++) { //>0  0~127
                    iLUT[i]=(BYTE)(i);
                }
                for(i=128;i<gray;i++) { //<0 -1~-128
                    iLUT[i]=(i-256);
                }
                //2. inverse at gray 165
                for(i=0;i<256;i++) { //37~293
                    iLUT[i]=165-iLUT[i];
                }
                //3. stretch in several sects
                for(i=0;i<256;i++) { //12~245
                    if( iLUT[i]<105) { //12~35
                        iLUT[i]=(short)(0.334*iLUT[i]);
                    }
                    else if( iLUT[i]<165) {//35~150
                        iLUT[i]=(short)(1.916*iLUT[i]-166);
                    }
                    else if( iLUT[i]<205) {//150~245
                        iLUT[i]=(short)(2.38*iLUT[i]-242);
                    }
                    else { //245~255
                        iLUT[i]=(short)(0.113*iLUT[i]+222);
                    }
                }
                //4. to lut in byts
                for(i=0;i<256;i++) {
                    bLUT[i][0]=(char)iLUT[i];
                    bLUT[i][1]=(char)iLUT[i];
                    bLUT[i][2]=(char)iLUT[i];
                }
                break;
            default:
                return 0;
        }
        okFillOutLUT(hBoard, (LPBYTE)bLUT, start, gray);

    }
    else { //for great than 8 bits lut
        WORD	wLUT[4096][3];
        short	num,gray;

        gray=(1<<bits);
        num=gray-1;
        switch (index) {
            case 0: //normal
                for(i=0;i<gray;i++) {
                    wLUT[i][0]=i;
                    wLUT[i][1]=i;
                    wLUT[i][2]=i;
                }
                break;
            case 1: //inverse
                for(i=0;i<=num;i++) {
                    wLUT[i][0]=(num-i);
                    wLUT[i][1]=(num-i);
                    wLUT[i][2]=(num-i);
                }
                break;
            default:
                return -1;
        }
        okFillOutLUT(hBoard, (LPBYTE)wLUT, start, gray);
    }

    return 1;
}

BOOL MainWindow::exFillInputLUT(HANDLE hBoard, short index)
{
    short	i,bits,start=0;

    //check how much bits LUT suppored
    bits=(short)okFillInputLUT(hBoard, nullptr, 0, 0);
    if(bits==-1)
        return -1; //not support

    if(	bits<=8) //for 8 bits lut
    {
        BYTE	bLUT[256][3];
        short	num=255,gray=256;

        switch (index) {
            case 0: //normal
                for(i=0;i<gray;i++) {
                    bLUT[i][0]=(BYTE)i;
                    bLUT[i][1]=(BYTE)i;
                    bLUT[i][2]=(BYTE)i;
                }
                break;
            case 1: //inverse
                for(i=0;i<num;i++) {
                    bLUT[i][0]=(BYTE)(num-i);
                    bLUT[i][1]=(BYTE)(num-i);
                    bLUT[i][2]=(BYTE)(num-i);
                }
                for(i=num;i<gray;i++) {
                    bLUT[i][0]=0;
                    bLUT[i][1]=0;
                    bLUT[i][2]=0;
                }
                break;
            default:
                return 0;
        }
        okFillInputLUT(hBoard, (LPBYTE)bLUT, start, gray);

    }
    else { //for great than 8 bits lut
        WORD	wLUT[4096][3]; //max for 12 bits
        short	gray;

        gray=(1<<bits);

        switch (index) {
            case 0: //normal
                for(i=0;i<gray;i++) {
                    wLUT[i][0]=i;
                    wLUT[i][1]=i;
                    wLUT[i][2]=i;
                }
                break;
            case 1: //inverse
                for(i=0;i<gray;i++) {
                    wLUT[i][0]=(gray-i-1);
                    wLUT[i][1]=(gray-i-1);
                    wLUT[i][2]=(gray-i-1);
                }
                break;
            default:
                return -1;
        }
        okFillInputLUT(hBoard, (LPBYTE)wLUT, start, gray);
    }

    return 1;
}

//--prnpic.c
int	MainWindow::exPrintReport(HWND hWnd, LPBITMAPINFOHEADER lpbi, LPBYTE lpdib)
{
    /*
    HDC		hPr;
    DOCINFO	docinfo;


    hPr = GetPrinterDC(hWnd);
    if (!hPr) {
        return (0);
    }

    lpAbortDlg =  MakeProcInstance((FARPROC)AbortDlg, hInst);
    lpAbortProc = MakeProcInstance((FARPROC)AbortProc, hInst);


    if (!SetAbortProc (hPr, (ABORTPROC)lpAbortProc))
        return 0;

    //2 startup
    docinfo.cbSize=sizeof(DOCINFO);
    docinfo.lpszDocName="prnfile";
    docinfo.lpszOutput=NULL;
    docinfo.lpszDatatype=NULL;
    docinfo.fwType=0;

    if (!StartDoc (hPr, &docinfo))
        return 0;
    if( !StartPage (hPr) )
        return 0;

    bAbort = FALSE; // Clears the abort flag


    // Create the Abort dialog box (modeless)
    //hAbortDlgWnd = CreateDialog((HINSTANCE)GetWindowLong(hWnd,GWL_HINSTANCE), "AbortDlg",
    hAbortDlgWnd = CreateDialog((HINSTANCE)GetWindowLong(hWnd,-6), "AbortDlg",
        hWnd, (DLGPROC)lpAbortDlg);

    if (!hAbortDlgWnd) {
        return (FALSE);
    }

    // Now show Abort dialog
    ShowWindow (hAbortDlgWnd, SW_NORMAL);

    // Disable the main window to avoid reentrancy problems
    EnableWindow(hWnd, FALSE);


    OutReport(hPr,lpbi,lpdib);

    //end print
    if( !EndPage (hPr) )
        return 0;
     // End the print operation.  Only send the ENDDOC if
     //   we didn't abort or error.
    if (!bAbort) //not abort yet
        if(!EndDoc (hPr) )
            return 0;
     // All done, clean up.

    EnableWindow(hWnd, TRUE);

    // Destroy the Abort dialog box
    DestroyWindow(hAbortDlgWnd);
    FreeProcInstance(lpAbortDlg);
    FreeProcInstance(lpAbortProc);

    DeleteDC(hPr);
    */

    return 1;
}

//--audio.c
extern exWAVEFILEHEADER waveheader;
extern QFile   hFile;

#if defined(Q_OS_LINUX)
    /* flags for wFormatTag field of WAVEFORMAT */
    #define WAVE_FORMAT_PCM     1
#endif

//create wav file by way of callback function of capture audio function
BOOL MainWindow::exCreateAudioFile2(HANDLE hAudio, LPSTR szFileName, long lSampFrq, short iSampBits, short iVolume)
{
    //set parameters
    okSetAudioParam(hAudio,AUDIO_SAMPLEFRQ,lSampFrq);
    okSetAudioParam(hAudio,AUDIO_SAMPLEBITS,iSampBits);
    okSetAudioParam(hAudio,AUDIO_INVOLUME,iVolume);
    okSetAudioParam(hAudio,AUDIO_CALLINTERVAL,300);

    //make header of wav file
    strncpy(waveheader.RIFF,"RIFF",4);
    strncpy(waveheader.WAVE,"WAVE",4);
    strncpy(waveheader.fmt,"fmt ",4);
    strncpy(waveheader.RIFF,"RIFF",4);
    waveheader.lHeadSize=0x10;
    strncpy(waveheader.data,"data",4);
    waveheader.wFormatTag=WAVE_FORMAT_PCM;//0x10;
    waveheader.nChannels=2;

    waveheader.nSamplesPerSec=lSampFrq;
    waveheader.nAvgBytesPerSec=lSampFrq*(iSampBits/8)*waveheader.nChannels;
    waveheader.nBlockAlign=4;
    waveheader.wBitsPerSample=iSampBits;

    //create file
    {
        hFile.setFileName(szFileName);
        if (!hFile.open(QIODevice::ReadWrite)) {
            return FALSE;
        }
    }

    waveheader.lFileLength=sizeof(exWAVEFILEHEADER);
    hFile.write((LPSTR)&waveheader,sizeof(exWAVEFILEHEADER));
    waveheader.lDataLength=0;

    //capture audio data
    okCaptureAudio(hAudio,BUFFER,(FARPROC)exWriteAudioData,0);


    return 1;
}

void MainWindow::exCaptureAudioFile(HANDLE hBoard, HWND hWnd)
{
    //test audio
    char	szFileName[]="test.wav";
    long	lSampFrq=44100;
    short	iSampBits=16;
    short	iVolume=120;


    //okCaptureTo(hBoard,SCREEN,0,0);

    hAudio=okOpenAudio(hBoard, 0);
    if(hAudio) {
        elapsedtimes=okGetTickCount();

//        hdc=GetDC(hWnd);
//        hPen=CreatePen(PS_SOLID,1,RGB(0,255,0));

        Sleep(2);
        lSampFrq = okSetAudioParam(hAudio, AUDIO_SAMPLEFRQ, -2);
        //ZeroMemory(lastaudio,1024);
        //capture audio and create file
        exCreateAudioFile2(hAudio,szFileName,lSampFrq,iSampBits,iVolume);

        FREEZE->setEnabled(true);

    }
}

BOOL MainWindow::exEndCaptureAudio(HANDLE hAudio)
{
    if(hAudio) {
        elapsedtimes=okGetTickCount()-elapsedtimes;
        QString str;
        str = QString(" Stoped ! captured times = %1 ms")
                .arg(elapsedtimes, 0, 10);
        setWindowTitle(str);

        okStopCaptureAudio(hAudio);
        okCloseAudio(hAudio);
        hAudio = nullptr;

        //way1 and way2 end file

    }

    return 1;
}

//--stream.c
//create wav file by way of using capture audio function directly
BOOL MainWindow::exCreateStreamFile3(HANDLE hStream, LPSTR szFileName)
{
    //set stream parameters

    //okSetStreamParam(hStream,STREAM_CALLINTERVAL,20);

//    elapsed=starttime=okGetTickCount();

    //make m2v file by call this function
    return (BOOL)(size_t)okCaptureStream(hStream,(TARGET)szFileName,(FARPROC)exMsgCallback,0);

    //return 1;
}

BOOL MainWindow::exCaptureStreamFile(HANDLE hBoard, HWND hWnd)
{
    //test audio
    char	szFileName[256]="d:\\test"; //also as test.mpg
    int		reslution=0; //d1, 3 sif
    //okCaptureTo(hBoard,SCREEN,0,0);

    if( !exM2VSaveName(szFileName) )
        return FALSE;

    if(!hStream)
        hStream=okOpenStream(hBoard, 0);

    if(hStream) {
        elapsedtimes=okGetTickCount();

#if defined(Q_OS_WIN32)
        reslution=GetPrivateProfileInt(L"Init", L"StreamResolution", 0, L"OkDemo.ini");
#else
        reslution=GetPrivateProfileInt("Init", "StreamResolution", 0, "OkDemo.ini");
#endif

        okSetStreamParam(hStream, STREAM_RESOLUTION, reslution); //SIF for c20b
        //okSetStreamParam(hStream, STREAM_BITRATEMODE, 1000); //
        okSetStreamParam(hStream, STREAM_BITRATEMODE, 3000); //for c20b
        okSetStreamParam(hStream, STREAM_MAXBITRATE, 9000); //for c20b

        //okSetStreamParam(hStream, STREAM_BITRATEMODE, 0); //for c20b
        //okSetStreamParam(hStream, STREAM_MAXBITRATE, 800); //for c20b, errer when small than 800

        //okSetStreamParam(hStream, STREAM_QUALITY, 50); //for rgb60c

        //okSetStreamParam(hStream, STREAM_MAXFILELEN, 1); //for c20b

        //capture video and audio and create file
        exCreateStreamFile3(hStream,szFileName); //

        //exEnableItem(hwnd, IDM_FREEZE, TRUE);

        return 1;

    }

    return 0;
}

BOOL MainWindow::exEndCaptureStream(HANDLE hStream)
{
    long	total;


    if(hStream) {

        //test pause
        //okStopCaptureStream(hStream,1); //pause
        //Sleep(5000);
        //okStopCaptureStream(hStream,2); //re-encode

        //Sleep(5000);

        //return 1;


        okStopCaptureStream(hStream,0);

        total=okStopCaptureStream(hStream,0);

//		okCloseStream(hStream);
        //hStream=0;
        //way1 and way2 end file


        elapsedtimes=okGetTickCount()-elapsedtimes;
        char str[200];
        sprintf(str," Stoped ! captured times = %d ms; Length=%d",elapsedtimes,total);
#if defined(Q_OS_WIN32)
        SetWindowTextA((HWND)winId(), str);
#endif
    }
    return 1;
}

//to open one board as current board to operate
HANDLE	MainWindow::exOpenCard()
{
    //RECT			rcVidRect;
    long	iNoBoards;
    //MEMORYSTATUS Status;
    long	error;
    char	str[120];

    //check ok boards sloted in PC
    //iNoBoards=okGetSlotBoard(NULL);
    iNoBoards=okGetImageDevice(nullptr,0);


    //check if pre_allocated buffer
    okGetAvailBuffer(nullptr, (MWORD *)&dwBufSize); //whole buffer
    //okGetBufferSize(hBoard,NULL,&dwBufSize); //available buffer for image

    //check if reg device driver for nt/2k/xp
    if( !okSetNTDriver(0) ) { // TRUE:registered already, -1:not nt system
        QMessageBox::about(this, "Warning", "There are not yet registered device service driver!");
    }
    else if( okSetNTDriver(0)==-1 ) { //win95/98
        if( !iNoBoards && (!okSetStaticVxD(0)) )
            QMessageBox::about(this, "Warning", "There are not yet registered vxd device driver!");
    }
    else {
        if(dwBufSize==0) //may be not reg driver, or no board, ...
            QMessageBox::about(this, "Warning", "There are no buffer pre-allocated !");
    }

    //				DialogBox(hInst, szSelectBox, hWnd, (FARPROC)exSelectDlgProc);





#if defined(Q_OS_WIN32)
    WritePrivateProfileString(L"Route", L"To Open Board", L"", L"OkDemo.ini");
#else
    WritePrivateProfileString("Route","To Open Board","","OkDemo.ini");
#endif
    //iCurrUsedNo=1//-1222;
    //open one specified board
    if( !(hBoard=okOpenBoard(&iCurrUsedNo)) ) {//set
    //if( !(hBoard=okOpenBoard("128.64.32.1")) ) {//set
    //if( !(hBoard=okOpenBoard("OK_C20B")) ) {//set
    //if( !(hBoard=okOpenBoard("OK_AM1360")) ) {//set
    //iCurrUsedNo=201360;
    //if( !(hBoard=okOpenBoard(&iCurrUsedNo)) ) {//set
            //if( !(hBoard=okOpenImageBoard(-1)) ) //select default

        error=okGetLastError();
        if(error==ERR_NOTFOUNDBOARD) {
            sprintf(str," ErrorCode=%d\n\n Not found correct 'Ok series Image Boards' in this Machine! ",error );
        //else if (error==ERR_NOTFOUNDVXDDRV)
        //	sprintf(str," ErrorCode=%d\n\n Failed to open device vxd/service driver for Ok Image Board!",error);
        }
        else {
            sprintf(str," ErrorCode=%d\n\n Open selected board failed! ",error);
        }
        QMessageBox::about(this, "Error !", str);

    }
    else {
        error=okGetLastError();
        if (error==ERR_NOTFOUNDVXDDRV) {
            sprintf(str," ErrorCode=%d\n\n Failed to open device driver for Ok Image Board!",error);
            QMessageBox::about(this, "Warning !", str);
        }
    }

    //just starting, then allocated memory for dib
    {
        //GlobalMemoryStatus(&Status);

        //determaine how many memory will be allocated as user memory
        //dwMaxMemSize=Status.dwTotalPhys-dwBufSize-32*(1<<20); //I'd keep min. 32M for window system
        //if(dwMaxMemSize<=0)
        //	dwMaxMemSize=5*(1<<20); //5M
        //else if( dwMaxMemSize>Status.dwAvailPhys )
        //	dwMaxMemSize=Status.dwAvailPhys/2;

        //if( dwMaxMemSize<5*(1<<20) ) //need 5M at least for user memory
        //	dwMaxMemSize=max(5*(1<<20),Status.dwAvailPhys);
        exAllocateDIBMemory(hBoard);

        if(!hBoard)
            return nullptr;

    }


    InitSetMenu(hBoard);


    return hBoard;
}

int MainWindow::exCloseThisBoard(HANDLE hBoard)
{
    DWORD	bSave=0;
    //okSetVideoParam(hBoard,VIDEO_BRIGHTNESS,10);
    //okSaveInitParam(hBoard,2);
//    SendMessage(hwndPlay,WM_COMMAND,IDCANCEL,0);

    okSetConvertParam(hBoard,CONVERT_UPRIGHT,0); //un
    okSetConvertParam(hBoard,CONVERT_MIRROR,0);

    if(hAudio) {//audio opened
        okCloseAudio(hAudio);
        hAudio = nullptr;
    }
    if(hStream) {//stream opened
        okCloseStream(hStream);
        hStream = nullptr;
    }
    if(hLoopThread) {
        TerminateThread(hLoopThread,0);
        hLoopThread = nullptr;
    }

    //restore interrupt set
    okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,intrMISCCONTROL|
            (okSetCaptureParam(hBoard,CAPTURE_MISCCONTROL,-1)&~0x3));
#if defined(Q_OS_LINUX)
    okCloseBoard(hBoard);
#else
    if( okSetDeviceParam(hBoard,DEVICE_GETPARAMCHANGED,0)>0) {
        int result=QMessageBox::question(this,"QtDemo","save param?",
                 QMessageBox::Yes|QMessageBox::No,//|QMessageBox::Cancel,
                 QMessageBox::Yes);
            if(result==QMessageBox::Yes)
            {
                bSave = true;
            }
            else {
                bSave = false;
            }
    }
    okCloseBoardEx(hBoard,bSave);

#endif

    exStopMulCapture(); //to stop in case mul-capturing

    return 1;
}

long MainWindow::exSetDataToDIB(TARGET src, short start, LPBITMAPINFOHEADER lpbi, LPBYTE lpdib)
{
    BLOCKINFO	blk;

    if(lpbi) {
        ZeroMemory(&blk,sizeof(BLOCKINFO));
        //get image to app buffer from tgt
        blk.lpBits=lpdib;
        blk.iWidth=(short)lpbi->biWidth;
        blk.iHeight=-(short)lpbi->biHeight;
        blk.iBitCount=lpbi->biBitCount;
        blk.lBlockStep=(short)lpbi->biSizeImage; //must be set
        blk.iHiStep=HIWORD(lpbi->biSizeImage);

        if(lpbi->biCompression==BI_BITFIELDS ) {//
            DWORD	*lpMask;
            lpMask=(DWORD *)((LPBYTE)lpbi+lpbi->biSize);
            if(lpMask[1]==0x03e0) //555
                blk.iFormType=FORM_RGB555;
            else if(lpbi->biBitCount==16)
                blk.iFormType=FORM_RGB565;
        }

        return okConvertRect(hBoard,(TARGET)&blk,0,src,start,1);
    }
}

//demo to get and save init set
BOOL MainWindow::exGetInitSet(BOOL bScrnCenter)
{
#if defined(Q_OS_WIN32)
    bScrnCenter=GetPrivateProfileInt(L"Init", L"Centerize", 1, L"OkDemo.ini");
    bMatchClient=GetPrivateProfileInt(L"Init", L"MatchClient", 0, L"OkDemo.ini");

    iClientWidth=GetPrivateProfileInt(L"Init", L"ClientWidth", 0, L"OkDemo.ini");
    iClientHeight=GetPrivateProfileInt(L"Init", L"ClientHeight", 0, L"OkDemo.ini");
#else
    bScrnCenter=GetPrivateProfileInt("Init", "Centerize", 1, "OkDemo.ini");
    bMatchClient=GetPrivateProfileInt("Init", "MatchClient", 0, "OkDemo.ini");

    iClientWidth=GetPrivateProfileInt("Init", "ClientWidth", 0, "OkDemo.ini");
    iClientHeight=GetPrivateProfileInt("Init", "ClientHeight", 0, "OkDemo.ini");
#endif
    if(bScrnCenter)
        ui->CENTERSCREEN->setChecked(true);
    else
        ui->CENTERSCREEN->setChecked(false);


    if(bMatchClient)
        ui->MATCHCLIENT->setChecked(true);
    else
        ui->MATCHCLIENT->setChecked(false);


    //bMakeMirror=GetPrivateProfileInt("Init","MakeMirror",0,"OkDemo.ini");
    //if(bMakeMirror&1)
    //	CheckMenuItem(hMenu,IDM_MAKEXMIRROR,MF_BYCOMMAND|MF_CHECKED);
    //else
    //	CheckMenuItem(hMenu,IDM_MAKEXMIRROR,MF_BYCOMMAND|MF_UNCHECKED);
    //if(bMakeMirror&2)
    //	CheckMenuItem(hMenu,IDM_MAKEYMIRROR,MF_BYCOMMAND|MF_CHECKED);
    //else
    //	CheckMenuItem(hMenu,IDM_MAKEYMIRROR,MF_BYCOMMAND|MF_UNCHECKED);


    return bScrnCenter;
}

BOOL MainWindow::exSaveInitSet(BOOL bScrnCenter)
{
#if defined(Q_OS_WIN32)
    wchar_t	string[20];

    wsprintf(string, L"%d", bScrnCenter);
    WritePrivateProfileString(L"Init", L"Centerize", string, L"OkDemo.ini");

    wsprintf(string, L"%d", bMatchClient);
    WritePrivateProfileString(L"Init", L"MatchClient", string, L"OkDemo.ini");

    wsprintf(string, L"%d", bMakeMirror);
    WritePrivateProfileString(L"Init", L"MakeMirror", string, L"OkDemo.ini");

    wsprintf(string, L"%d", iClientWidth);
    WritePrivateProfileString(L"Init", L"ClientWidth", string, L"OkDemo.ini");
    wsprintf(string, L"%d", iClientHeight);
    WritePrivateProfileString(L"Init", L"ClientHeight", string, L"OkDemo.ini");
#else
    char	string[20];

    sprintf(string,"%d",bScrnCenter);
    WritePrivateProfileString("Init","Centerize",string,"OkDemo.ini");

    sprintf(string,"%d",bMatchClient);
    WritePrivateProfileString("Init","MatchClient",string,"OkDemo.ini");

    sprintf(string,"%d",bMakeMirror);
    WritePrivateProfileString("Init","MakeMirror",string,"OkDemo.ini");

    sprintf(string,"%d",iClientWidth);
    WritePrivateProfileString("Init","ClientWidth",string,"OkDemo.ini");
    sprintf(string,"%d",iClientHeight);
    WritePrivateProfileString("Init","ClientHeight",string,"OkDemo.ini");
#endif

    return 1;
}

//get size of one frame of image in buffer
long MainWindow::exGetFrameSize(HANDLE hBoard)
{
    long	bits,size;

    size=okSetCaptureParam(hBoard,CAPTURE_BUFBLOCKSIZE,-1);
    bits=okSetCaptureParam(hBoard,CAPTURE_BUFRGBFORMAT,-1);
    size=(HIWORD(bits))/8*LOWORD(size)*HIWORD(size);

    return size;
}

void MainWindow::exDisplayDIBInfo(void)
{
    char	string[200];
    LPSTR	lpstr;

#if defined(Q_OS_WIN32)
    GetWindowTextA((HWND)winId(),string,120);
#endif
    lpstr=strchr(string,'-');
    if(!lpstr)
        lpstr=string+strlen(string);
    sprintf(lpstr,"- %d x %d x %d",lpbi->biWidth,lpbi->biHeight,lpbi->biBitCount);
    if(lpbi->biWidth)
        setWindowTitle(string);
}


//--demo to set color/gray bands to buffer and dib
BOOL MainWindow::exSetTestBands(LPBYTE lpLinear, short bits, short height, long stride)
{
    //set gray and color bands to dib and buffer 0
       int			i,j,k;
       short		offx,offy,step=bits/8;
       short		bwid=32*bits/8, halfx=stride/2/bits*bits,endx=stride-step+1;


       //set bands.  suppose buffer as format of 24 bits

       if(!lpLinear)
           return FALSE;

       for(i=0;i<height;i++)  //clear whole zero
           FillMemory(lpLinear+stride*i,stride,0);//i);

       offy=qMin(256,height/2);  // down half section (gray level section)
       for(i=offy;i<height;i+=1) {
           FillMemory(lpLinear+stride*i,stride/2,i-offy); //left half lines

           for(k=0,j=stride/2;j<stride;j+=24*4,k+=32) { //right half
               if(k==256)
                   k=255;
               FillMemory(lpLinear+stride*i+j,24*4,k);  //k is value
           }
       }

       //for(i=0;i<height;i++)  //clear zero
       //	for(j=0;j<stride;j+=2) {
       //		*(WORD *)(lpLinear+stride*i+j)=i+1;
       //	}


       //-------------------------------------------------------

       offy=0;	//up half section (color bands section)
       //1black
       for(i=offy;i<offy+32;i++) {		//right half lines
           for(j=halfx;j<=endx;j+=step)
               *(DWORD *)(lpLinear+stride*i+j)=0x0;
       }
       offy=i;

       //1black
       offx=0;
       for(i=0;i<qMin(256,height/2);i++) {	//left half lines
           for(j=offx;j<offx+bwid;j+=step) {
               *(DWORD *)(lpLinear+stride*i+j)=0x0;
           }
       }

       //2blue
       for(i=offy;i<offy+32;i++) {
           for(j=halfx;j<endx;j+=step)
               *(DWORD *)(lpLinear+stride*i+j)=0xff;
       }
       offy=i;

       //2blue
       offx+=bwid;
       for(i=0;i<qMin(256,height/2);i++) {
           for(j=offx;j<offx+bwid;j+=step) {
               *(DWORD *)(lpLinear+stride*i+j)=0xff;
           }
       }

       //3red
       for(i=offy;i<offy+32;i++) {
           for(j=halfx;j<endx;j+=step)
               *(DWORD *)(lpLinear+stride*i+j)=0xff0000;
       }
       offy=i;

       //3green
       offx+=bwid;
       for(i=0;i<qMin(256,height/2);i++) {
           for(j=offx;j<offx+bwid;j+=step) {
               *(DWORD *)(lpLinear+stride*i+j)=0xff00;
           }
       }

       //4magenta
       for(i=offy;i<offy+32;i++) {
           for(j=halfx;j<endx;j+=step)
               *(DWORD *)(lpLinear+stride*i+j)=0xff00ff; ;
       }
       offy=i;

       //4red
       offx+=bwid;
       for(i=0;i<qMin(256,height/2);i++) {
           for(j=offx;j<offx+bwid;j+=step) {
               *(DWORD *)(lpLinear+stride*i+j)=0xff0000;
           }
       }

       //5green
       for(i=offy;i<offy+32;i++) {
           for(j=halfx;j<endx;j+=step)
               *(DWORD *)(lpLinear+stride*i+j)=0xff00;
       }
       offy=i;

       //5cyan
       offx+=bwid;
       for(i=0;i<qMin(256,height/2);i++) {
           for(j=offx;j<offx+bwid;j+=step) {
               *(DWORD *)(lpLinear+stride*i+j)=0xffff;
           }
       }

       //6cyan
       for(i=offy;i<offy+32;i++) {
           for(j=halfx;j<endx;j+=step)
               *(DWORD *)(lpLinear+stride*i+j)=0xffff;
       }
       offy=i;

       //6magenta
       offx+=bwid;
       for(i=0;i<qMin(256,height/2);i++) {
           for(j=offx;j<offx+bwid;j+=step) {
               *(DWORD *)(lpLinear+stride*i+j)=0xff00ff;
           }
       }

       //7yellow
       for(i=offy;i<offy+32;i++) {
           for(j=halfx;j<endx;j+=step)
               *(DWORD *)(lpLinear+stride*i+j)=0xffff00;
       }
       offy=i;
       //7yellow
       offx+=bwid;
       for(i=0;i<qMin(256,height/2);i++) {
           for(j=offx;j<offx+bwid;j+=step) {
               *(DWORD *)(lpLinear+stride*i+j)=0xffff00;
           }
       }

       //8white
       for(i=offy;i<offy+32;i++) {
           for(j=halfx;j<endx;j+=step)
               *(DWORD *)(lpLinear+stride*i+j)=0xffffff;
       }
       offy=i;
       //8white
       offx+=bwid;
       for(i=0;i<qMin(256,height/2);i++) {
           for(j=offx;j<offx+bwid;j+=step) {
               *(DWORD *)(lpLinear+stride*i+j)=0xffffff;
           }
       }

       //repeat
   //	offx+=bwid;
   //	while(stride/2>offx) {//remain
   //		for(i=0;i<min(256,height/2);i++) {
   //			memcpy(lpLinear+stride*i+offx,lpLinear+stride*i,stride/2-offx);
   //		}
   //		offx+=bwid*8;
   //	}

       offx+=bwid;
       offx+=bwid/2; // blue vertical line
       for(i=0;i<qMin(256,height/2);i++) {
           *(DWORD *)(lpLinear+stride*i+offx)=0xff;
       }

       offx+=bwid/2; // green vertical line
       for(i=0;i<qMin(256,height/2);i++) {
           *(DWORD *)(lpLinear+stride*i+offx)=0xff00;
       }

       offx+=bwid/2; //red vertical line
       for(i=0;i<qMin(256,height/2);i++) {
           *(DWORD *)(lpLinear+stride*i+offx)=0xff0000;
       }

       offx+=bwid/2; //cyan vertical line
       for(i=0;i<qMin(256,height/2);i++) {
           *(DWORD *)(lpLinear+stride*i+offx)=0xffff00;
       }

       offx+=bwid/2; //magenta vertical line
       for(i=0;i<qMin(256,height/2);i++) {
           *(DWORD *)(lpLinear+stride*i+offx)=0xff00ff;
       }

       offx+=bwid/2; //yellow vertical line
       for(i=0;i<qMin(256,height/2);i++) {
           *(DWORD *)(lpLinear+stride*i+offx)=0xffff00;
       }

       offx+=bwid/2; //white vertical line
       for(i=0;i<qMin(256,height/2);i++) {
           *(DWORD *)(lpLinear+stride*i+offx)=0xffffff;
       }

       //repeat
       offx+=bwid;
       while(stride/2>offx) {//remain
           for(i=0;i<qMin(256,height/2);i++) {
               memcpy(lpLinear+stride*i+offx,lpLinear+stride*i+bwid,stride/2-offx);
           }
           offx+=bwid*8;
       }


       //set test vert line raster
       //for(i=0;i<height;i++)  //clear zero
       //	FillMemory(lpLinear+stride*i,stride,0);//i);
       //for(j=0;j<stride;j+=3*8) {
       //	for(i=0;i<height;i++) {
       //		*(DWORD *)(lpLinear+stride*i+j)=0xffffff;
       //	}
       //}

       //set test horz line raster
       //for(i=0;i<height;i+=8) {
       //	FillMemory(lpLinear+stride*i,stride,255);
       //}


       return 1;
}

long MainWindow::exSetToWndRectEx(HANDLE hBoard, long lMode)
{
    short	iHBar,iVBar;
    DWORD	dwWinStyle;
    long	lSize, lHScrMax,lVScrMax;
    RECT	rect;
    SCROLLINFO	scrinfo;
/*
    //get client rect
    GetClientRect(hWnd,&rect);

    iHBar=GetSystemMetrics (SM_CYHSCROLL);
    iVBar=GetSystemMetrics (SM_CXHSCROLL);
    dwWinStyle = GetWindowLong (hWnd, GWL_STYLE);
    if (dwWinStyle & WS_HSCROLL)
        rect.bottom += iHBar;
    if (dwWinStyle & WS_VSCROLL)
        rect.right  += iVBar;

    if( (sampwidth <= rect.right) &&
            (sampheight<=rect.bottom) ) { //no bars
        lHScrMax= 0;
        lVScrMax= 0;
    } else if( (sampwidth+iVBar)<=rect.right ) { //surely no h bar
        lHScrMax= 0;
        rect.right -=iVBar;
        lVScrMax= sampheight-rect.bottom;
    } else if( (sampheight+iHBar)<=rect.bottom ) { //surely no v bar
        lHScrMax= sampwidth - rect.right;
        lVScrMax= 0;
        rect.bottom -=iHBar;
        //horzstart=lHScrMax+1;
    } else { //with two bars
        rect.right -=iVBar;
        rect.bottom -=iHBar;
        lHScrMax= sampwidth  - rect.right;
        lVScrMax= sampheight  - rect.bottom;
    }
//#ifdef _DEBUG

    scrinfo.cbSize=sizeof(SCROLLINFO);
    scrinfo.fMask=SIF_PAGE|SIF_RANGE|SIF_POS;// SIF_ALL;

    //------------------horz
    GetScrollInfo(hWnd, SB_HORZ, &scrinfo);
    scrinfo.nMin=0;
    scrinfo.nPage=1;
    scrinfo.nTrackPos=0;
    scrinfo.nMax=0;

    if((lHScrMax>0) && (bMatchClient==0) ) {  //with scroll bar, and no stretch
        scrinfo.nPage=rect.right&~1;
        //WndScrollControl(hWnd, message, LOWORD(wParam),HIWORD(wParam), bActive);
        scrinfo.nMax=scrinfo.nPage+lHScrMax-1;
    }
    horzstart=min(horzstart,scrinfo.nMax-scrinfo.nPage+1);
    scrinfo.nTrackPos=scrinfo.nPos=horzstart;
    SetScrollInfo(hWnd, SB_HORZ, &scrinfo,TRUE);//redraw

    //{
    //	HDC		hdc;
    //	char	str[120];

    //	hdc=GetDC(NULL);
    //	sprintf(str,"%d,%d,%d,%d     ",scrinfo.nMax,scrinfo.nPage,scrinfo.nPos,horzstart);
    //	TextOut(hdc,0,0,str,strlen(str));
    //	ReleaseDC(NULL,hdc);

    //}

    //----------vert
    GetScrollInfo(hWnd, SB_VERT, &scrinfo);
    scrinfo.nMin=0;
    scrinfo.nPage=1;
    scrinfo.nTrackPos=0;
    scrinfo.nMax=0;
    if((lVScrMax>0) && (bMatchClient==0) ) {  //with scroll bar, and no stretch
        scrinfo.nPage=rect.bottom;
        scrinfo.nMax=scrinfo.nPage+lVScrMax-1;
    }
    vertstart=min(vertstart,scrinfo.nMax-scrinfo.nPage+1);
    scrinfo.nTrackPos=scrinfo.nPos=vertstart;
    SetScrollInfo(hWnd, SB_VERT, &scrinfo,TRUE); //redraw

    okSetCaptureParam(hBoard,CAPTURE_SCROFFSETVID,MAKELONG(horzstart,vertstart));

//#endif

    okSetToWndRect(hBoard, hWnd); //set hWnd of main process to hBoard
    if(lMode==1) { //if set to center
        //short		width,height;
        //get available rect size
        //lSize=okSetVideoParam(hBoard,VIDEO_AVAILRECTSIZE,-1);
        //or set to expected size
        //sampwidth=LOWORD(lSize);
        //sampheight=HIWORD(lSize);

        //if( okGetCaptureStatus(hBoard,0) ) { //capturing
        //	width=sampwidth;
        //	height=sampheight;
        //}
        //else { //frozen status
        //	width=(short)lpbi->biWidth;			//???
        //	height=(short)lpbi->biHeight;
        //}
        if( (rect.right) > sampwidth )
            rect.left=( (rect.right - sampwidth )/2 ) ;//& (~0x3);
        if( (rect.bottom) > sampheight )
            rect.top=( (rect.bottom - sampheight )/2);
    }

    //make offset x,y
    lSize=MAKELONG(rect.left,rect.top);
    rect.right=rect.left+min(sampwidth,rect.right);
    rect.bottom=rect.top+min(sampheight,rect.bottom);

    //map to absolute screen rect
    MapWindowPoints(hWnd,HWND_DESKTOP,(LPPOINT)&rect,2);

    if(bMatchClient==0) //need not to match client rect
        okSetTargetRect(hBoard, SCREEN, &rect);
*/
    return lSize;
}


long MainWindow::exAllocateDIBMemory(HANDLE hBoard )
{
    char	flag=0,string[100];
    DWORD	dwDIBMemSize;
    RECT	rcBuf;


    //allocate memory for bi and dib to paint current window
    if(lpbi==nullptr) { //just one time
        lpbi=(LPBITMAPINFOHEADER)GlobalAlloc(GPTR,2048);
        m_PaintView->SetBI(lpbi);
    }

    if(!hBoard)
        return 0;

    //get current buffer size
    SetRect(&rcBuf,0,0,-1,-1);
    okSetTargetRect(hBoard,BUFFER,&rcBuf);
    dwDIBMemSize =(((rcBuf.right+3)*4/4)*rcBuf.bottom*4) ; // 32

    if( dwDIBMemSize>dwDIBCurrSize ) {
        dwDIBCurrSize=dwDIBMemSize;
        GlobalFree(lpdib);
        lpdib = nullptr;
    }
    if( lpbi->biSizeImage>dwDIBCurrSize) {
        dwDIBCurrSize=lpbi->biSizeImage;
        if(lpdib)
            GlobalFree(lpdib);
        lpdib = nullptr;
    }


    if(lpdib==nullptr) {
        lpdib=(LPBYTE)GlobalAlloc(GPTR,dwDIBCurrSize);
        flag=1; //changed
    }
    m_PaintView->SetDIB(lpdib);

    if(!lpdib) { //no allocated
        sprintf(string,"Allocate DIB memory failed !");
        QMessageBox::about(this, "Warning", string);
        return FALSE;
    }


    //GlobalMemoryStatus(&Status);
    if((flag==1) ) //re-allocated
    { //changed or init;
        if( dwDIBCurrSize<= (4*(1<<20)) ) { //for small than 4M
            dwMaxMemSize=dwDIBCurrSize*4;//16M
        }else { //big dib
            dwMaxMemSize=dwDIBCurrSize;//
        }
        if(lpMemory)
            GlobalFree(lpMemory);

        lpMemory=(LPBYTE)GlobalAlloc(GPTR,dwMaxMemSize);
    }

    return (dwDIBCurrSize); //total allocated
}

void MainWindow::closeEvent(QCloseEvent *event)
{

    exCloseThisBoard(hBoard);
    hBoard = nullptr;

    GlobalFree((HGLOBAL)lpbi);
    lpbi = nullptr;
    GlobalFree((HGLOBAL)lpdib);
    lpdib = nullptr;
    GlobalFree((HGLOBAL)lpMemory);
    lpMemory = nullptr;

    exSaveInitSet(bScrnCenter);

}
