﻿#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>

#if defined(Q_OS_WIN64)
    #include "Windows.h"
    #include "Okapi64.h"
#elif defined(Q_OS_WIN32)
    #include "windows.h"
    #include "Okapi32.h"
#else
    #include "../share/winsim.h"
    #include "../share/okapi64.h"
#endif

#include <paintview.h>


typedef struct _wavfileheader {
    char	RIFF[4];
    long	lFileLength;
    char	WAVE[4];
    char	fmt[4];
    long	lHeadSize;

    //PCMWAVEFORMAT pcmheader;
    WORD	wFormatTag;		//format type
    WORD	nChannels;		//number of channels (i.e. mono, stereo...)
    DWORD	nSamplesPerSec;	//sample rate
    DWORD	nAvgBytesPerSec;//for buffer estimation
    WORD	nBlockAlign;	//block size of data
    WORD	wBitsPerSample;	//Number of bits per sample of mono data
    char	data[4];
    long	lDataLength;
} exWAVEFILEHEADER;


namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:
    bool eventFilter(QObject *obj, QEvent *event);

private:
    Ui::MainWindow *ui;

private slots:
    //file
    void on_LOADFILE_triggered();
    void on_SAVESCREEN_triggered();
    void on_SAVEBUFFER_triggered();
    void on_COPYTOCLIPB_triggered();
    void on_SETTOZERO_triggered();
    void on_SETTOBAND_triggered();
    void on_ZEROALLBUFFER_triggered();
    void on_PRINTPIC_triggered();
    void on_LOADCONFIGFILE_triggered();
    void on_SAVECONFIGFILE_triggered();
    void on_EXITOKDEMO_triggered();

    //option
    void on_SETUPVIDEO_triggered();
    void on_CAPSEQUENCE_triggered();
    void on_FILLINGLUT_triggered();
    void on_CENTERSCREEN_triggered();
    void on_ENABLEMASK_triggered();
    void on_MAKEXMIRROR_triggered();
    void on_MAKEYMIRROR_triggered();
    void on_UPTORIGHT_triggered();
    void on_UPTOLEFT_triggered();
    void on_NORMAINLUT_triggered();
    void on_INVERSEINLUT_triggered();
    void on_MATCHCLIENT_triggered();
    void on_SETMATCHSIZE_triggered();
    void on_REPLAYBUFFER_triggered();
    void on_REPLAYMEMORY_triggered();
    void on_REPLAYFILE_triggered();
    void on_BUFFERTOSCREEN_triggered();
    void on_SCREENTOBUFFER_triggered();
    void on_BUFFER0TOBUFFER1_triggered();
    void on_BUFFER1TOBUFFER0_triggered();
    void on_BUFFERTOFRAME_triggered();
    void on_FRAMETOBUFFER_triggered();
    void on_FRAMETOSCREEN_triggered();
    void on_SELECTCARD_triggered();

    //Capture
    void on_CAPTOBUFFER_triggered();
    void on_LOOPTOBUFFER_triggered();
    void on_SEQCAPTOBUF_triggered();
    void on_LOOPSEQCAPTOBUF_triggered();
    void on_CAPTOMEMORY_triggered();
    void on_CAPTOSEQFILE_triggered();
    void on_CAPTOINDIRECT_triggered();
    void on_INTERTOINDIRECT_triggered();
    void on_CAPTOPSEUDO_triggered();
    void on_CAPTODIRECT_triggered();
    void on_CAPTOFOREVER_triggered();
    void on_CONTTOBUFFER0_triggered();
    void on_CONTTOBUFFER1_triggered();
    void on_CAPTOFRAME_triggered();
    void on_MULCHANCAP_triggered();
    void on_MULCHANCAPSUB_triggered();
    void on_ASYNCMULCAP_triggered();
    void on_SYNCMULCAP_triggered();
    void on_CAPTUREAUDIO_triggered();
    void on_CAPTUREMPEG2_triggered();

    //Display
    void on_DISPFROMBUFFER_triggered();
    void on_LOOPFROMBUFFER_triggered();
    void on_INTERFROMBUFFER_triggered();
    void on_DISPFROMMEMORY_triggered();
    void on_DISPFROMFILE_triggered();
    void on_CAPTOMONITOR_triggered();
    void on_DISPFROMFRAME_triggered();
    void on_NORMALLUT_triggered();
    void on_INVERSELUT_triggered();
    void on_ABSOLUTELUT_triggered();
    void on_ABSOLUTEDSA_triggered();


    //Freeze
    void OnFreezeClicked();

    //Active
    void OnActiveClicked();

    //SingleCapTo
    void OnSingleCapToClicked();

    //help
    void OnTestVGACard();
    void OnTestDrawWnd();
    void OnTestMainWnd();
    void OnDisableActive(bool);
    void OnDisableSyncVert();
    void OnSetAllocBuf();
    void OnLockBufSize();
    void OnRegDevDriver();
    void OnSignalClicked();
    void OnExitTrigger();
    void OnAboutClicked();
    void closeEvent(QCloseEvent *event);

private:
    PaintView *m_PaintView;

    QAction     *FREEZE;
    QAction     *ACTIVE;

    QAction     *DISABLEACTIVE;
    QAction     *DISABLESYNCVERT;
    QAction     *REGDEVDRIVER;

    void CreateMenu();
private:
    /*-----------------------------------------------------------------------------
    *       Global variables for board
    */

    HANDLE		hLoopThread;
    HANDLE		hBoard;
    BOOL		bActive=0;
    short		total=2;
    MLONG		iCurrUsedNo=-1;//
    long		intrMISCCONTROL; //b0:1

    short		bScrnCenter=0; //non-centerize
    long		lScrnOffset;
    WORD		sampwidth,sampheight;		//width x height
/*    WORD		horzstart,vertstart;			//horz start, vert start
    */
    short		bMatchClient=0; // match client rect
    short		bMakeMirror=0; //no mirror
    int			iClientWidth;
    int			iClientHeight;

    short		bMaskMode=0; // mask disable
/*
    LPSTR		seqfile="ok.seq";*/
    DWORD		dwMaxMemSize;
    DWORD		dwDIBCurrSize=1024*1024*4; //
    long		dwBufSize;
/*
    long		lNumImage=NUMINFILE;
*/

    LPBYTE				lpMemory = NULL; //for memory to capture by buffer
/*    BLOCKINFO			blkinfo;


    LPDIBINFO			lpDibPlay;	//for ReplayDlg*/

    HWND				hwndPlay; //for ReplayDlgEx

/*    WORD				lENG=0;

    long				lCameraCode=-1;		//OkCamera
    */

/*
    //-------------------------------------------------------------------------------------------

    extern LPBITMAPINFOHEADER	lpbi;
    extern HWND	hwnd;*/
//    HDC		hdc;
//    HPEN	hPen;
/*    char	lastaudio[2048];*/
    long	elapsedtimes;
//    char	str[100];

    /*-----------------------------------------------------------------------------
    *       Extern functions ( in other files)
    */


    //OkCamera
    //SetCamera.c
/*    BOOL SetCameraVideoSignal(HANDLE hBoard,long lCameraCode,int videotype, int scanlines, int fieldfrq);
    //SetChan.c
    BOOL AutoSetChan(HANDLE hBoard);//setchan return=TRUE(exist)	FALSE(not exist)
*/
    //--file.c
    BOOL exM2VSaveName(char *szFileName);
    BOOL exGetSaveName(char *szFileName);
    BOOL exGetLoadName(char *szFileName);

    BOOL exLoadImageFile(HANDLE hBoard);
    BOOL exSaveScreenFile(HANDLE hBoard);
    BOOL exSaveBufferFile(HANDLE hBoard);

    BOOL exLoadCfgFile(HANDLE hBoard);
/*    BOOL exSaveCfgFile(HANDLE hBoard, char *szFileName);


    //--bitmap.c
    BOOL exGetBitmapData(HANDLE hBoard, TARGET tgt, short start, LPBITMAPINFOHEADER lpbi, LPBYTE lpdib);*/
    BOOL exGetBitmapHeader(HANDLE hBoard, TARGET src, LPBITMAPINFOHEADER lpbi);
/*    HPALETTE  exSetPaletteToWnd(HANDLE hBoard, HDC hDC);*/
    BOOL exMoveToClipboard( LPBITMAPINFOHEADER lpbi, LPBYTE lpdib );
    BOOL exConvertBitmap(LPBITMAPINFOHEADER lpbi,LPSTR lpdib);

/*    HANDLE exCreateWatching(HANDLE hBoard, HWND hWnd, long lBufNo);
    HANDLE exCreateLoopCapture(HANDLE hBoard, HANDLE hWnd);

    //--mulcap.c
    BOOL exMultiChannCapture(HANDLE hBoard, HWND hWnd, BOOL bScrnCenter);
    BOOL exMultiChannSubArea(HANDLE hBoard, HWND hWnd, BOOL bScrnCenter);
    BOOL exAsyncMulCapture(HWND hWnd, BOOL bScrnCenter);
    BOOL exSyncMulCapture(HWND hWnd, BOOL bScrnCenter);*/
    int	 exStopMulCapture();


    //--extra.c
    BOOL exDemoMask(HANDLE hBoard, BOOL bMaskMode);
    BOOL exFillOutLUT(HANDLE hBoard, short index);
    BOOL exFillInputLUT(HANDLE hBoard, short index);


    //--prnpic.c
    int	exPrintReport(HWND hWnd, LPBITMAPINFOHEADER lpbi, LPBYTE lpdib);


    //--audio.c
    BOOL exCreateAudioFile2(HANDLE hAudio, LPSTR szFileName, long lSampFrq, short iSampBits, short iVolume);
    void exCaptureAudioFile(HANDLE hBoard, HWND hWnd);
    BOOL exEndCaptureAudio(HANDLE hAudio);

    //--stream.c
    BOOL exCreateStreamFile3(HANDLE hStream, LPSTR szFileName);
    BOOL exCaptureStreamFile(HANDLE hBoard, HWND hWnd);
    BOOL exEndCaptureStream(HANDLE hStream);

    /*-----------------------------------------------------------------------------
    *       Local functions ( in this file)
    */

    //----open and set-------
    HANDLE	exOpenCard();
    int exCloseThisBoard(HANDLE hBoard);

    long exSetDataToDIB(TARGET src, short start, LPBITMAPINFOHEADER lpbi, LPBYTE lpdib);
    BOOL exGetInitSet(BOOL bScrnCenter);
    BOOL exSaveInitSet(BOOL bScrnCenter);
    long exGetFrameSize(HANDLE hBoard);
    void exDisplayDIBInfo(void);
/*
    //----get and set by dialog ------

    BOOL APIENTRY exSetNumImage(HWND hDlg,unsigned message,WORD wParam,LONG lParam);
    BOOL APIENTRY exSetClientSize(HWND hDlg,unsigned message,WORD wParam,LONG lParam);
    BOOL APIENTRY exSelectDlgProc(HWND hDlg,unsigned message,WORD wParam,LONG lParam);
    BOOL APIENTRY exSetAllocBuf(HWND hDlg,unsigned message,WORD wParam,LONG lParam);
    BOOL APIENTRY exSetLockBuffer(HWND hDlg,unsigned message,WORD wParam,LONG lParam);
    BOOL APIENTRY exAutoSetVideoParams(int videotype, int scanlines, int fieldfrq, int sample43);
    BOOL APIENTRY exSignalInfo(HWND hDlg,unsigned message,WORD wParam,LONG lParam);
    BOOL APIENTRY exAbout(HWND hDlg,unsigned message,WORD wParam,LONG lParam);
*/
    //----window process -------
    BOOL exSetTestBands(LPBYTE lpLinear, short bits, short height, long stride);
    long exSetToWndRectEx(HANDLE hBoard, long lMode);
/*    BOOL exDispPointValue(HANDLE hBoard, HWND hwnd, LPARAM lParam);
    */

    long exAllocateDIBMemory(HANDLE hBoard);
    BOOL InitSetMenu(HANDLE hBoard);






    //audio.c
    HANDLE	hAudio = nullptr;

    //stream.c
    HANDLE	hStream = nullptr;

    //mulcap.c
    int		NoCapture = 0;
    HANDLE	lphBoard[9] = {nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr};
//public:
 //   long exAllocateDIBMemory(HANDLE hBoard);
};

#endif // MAINWINDOW_H
